/*-----------------------------------------
   MAIN.C -- HMS Surprise Main module
   Version date: 02.12.11
  -----------------------------------------*/

#define  PI       3.1415927
#define  DEGTORAD PI/180.0

#include <windows.h>
#include <commctrl.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "ezfont.h"
#include "Surprise.h"
#include "Menu.h"

// Prototypes:
int WINAPI        WinMain (HINSTANCE, HINSTANCE, PSTR, int) ;
LRESULT CALLBACK  WndProc (HWND, UINT, WPARAM, LPARAM) ;
BOOL CALLBACK     AboutDlgProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  EditProc (HWND, UINT, WPARAM, LPARAM) ;
BOOL CALLBACK     HelmDlgProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  MapWndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  PropsProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  PlanWndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  SailWndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT           StatusMessage (HWND, WPARAM, LPARAM) ;
LRESULT CALLBACK  SternWndProc (HWND, UINT, WPARAM, LPARAM) ;
VOID CALLBACK     Timer1Proc (HWND, UINT, UINT, DWORD) ;
VOID CALLBACK     Timer2Proc (HWND, UINT, UINT, DWORD) ;
BOOL CALLBACK     WindDlgProc (HWND, UINT, WPARAM, LPARAM) ;

extern void    Calculate () ;
extern void    CanShadow () ;
extern void    CycleEvents () ;
extern void    DecrSail () ;
extern void    DerivedShipParameters () ;
extern void    DerivedSailParameters () ;
extern HFONT   EZCreateFont (HDC, char *, int, int, int, BOOL) ;
extern void    FileInitialize () ;
extern BOOL    FileOpenDlg (PSTR, PSTR) ;
extern BOOL    FileSaveDlg (PSTR, PSTR) ;
extern void    Focusser () ;
extern void    HelmAdjust () ;
extern void    IncrSail () ;
extern void    JibChange (int) ;
extern void    LoadConfiguration () ;
extern void    LoadDefault () ;
extern void    Map () ;
extern void    Pause () ;
extern void    PlanView () ;
extern void    PropertiesAccept () ;
extern void    PropertiesCancel () ;
extern void    PropertiesChange () ;
extern void    PropertiesPaint () ;
extern void    PropertiesSetUp () ;
extern void    SailPlan () ;
extern void    SaveConfiguration () ;
extern void    SetUpDisplay () ;
extern void    SelectSailParameter () ;
extern int     Sgn (float) ;
extern void    ShowActualJibAngle () ;
extern void    ShowMessage () ;
extern void    ShowParameters () ;
extern void    ShowStatusMessage () ;
extern void    SternView () ;
extern void    YardAdjust (int) ;
extern void    Yardsmen () ;
extern void    ZoomIn () ;
extern void    ZoomOut () ;

/*-----------------------------------------
   WinMain: Main windows procedure    
   Version date: 99.09.22
-----------------------------------------*/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
  {
  static char szAppName[] = "HMS Surprise" ;
  MSG         msg ;
  WNDCLASSEX  wndclass ;
    
  wndclass.cbSize        = sizeof (wndclass) ;
  wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
  wndclass.lpfnWndProc   = WndProc ;
  wndclass.cbClsExtra    = 0 ;
  wndclass.cbWndExtra    = 0 ;
  wndclass.hInstance     = hInstance ;
  wndclass.hIcon         = LoadIcon (hInstance, "Surprise") ;
  wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
  wndclass.hbrBackground = (HBRUSH) GetStockObject (LTGRAY_BRUSH) ;
  wndclass.lpszMenuName  = "Surprise" ;
  wndclass.lpszClassName = szAppName ;
  wndclass.hIconSm       = LoadIcon (hInstance, "Surprise") ;

  RegisterClassEx (&wndclass) ;

  hwnd = CreateWindow (szAppName, szAppName,
         WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL |WS_CLIPCHILDREN ,
         CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
         NULL, NULL, hInstance, NULL) ;

  if (!SetTimer (hwnd, 1, 460, (TIMERPROC) Timer1Proc))
    {
    MessageBox (hwnd, "Too many clocks or timers!", szAppName, MB_ICONEXCLAMATION | MB_OK) ;
    return FALSE ;
    }
  
  if (!SetTimer (hwnd, 2, 1, (TIMERPROC) Timer2Proc))
    {
    MessageBox (hwnd, "Too many clocks or timers!", szAppName, MB_ICONEXCLAMATION | MB_OK) ;
    return FALSE ;
    }

  ShowWindow (hwnd, SW_SHOWMAXIMIZED) ; 
    {
    RECT   rect ;
    GetClientRect (hwnd, &rect) ;

    xMax = rect.right-16 ;
    yMax = rect.bottom-16 ;  
    if (xMax>800)
      {
      // 1024*768 screen
      ShowWindow (hwnd, SW_NORMAL) ;
      MoveWindow(hwnd,0,0,816,616,TRUE) ;
      xMax=784 ;
      yMax=518 ;
      }
    if (xMax<700)
      {
      // 640*480 screen
      ShowWindow (hwnd, SW_NORMAL) ;
      MoveWindow(hwnd,0,0,624,398,TRUE) ;
      xMax=784 ;
      yMax=518 ;
      MessageBox (hwnd,"HMS Surprise is designed for 800*600 pixel screen resolution","Screen resolution",MB_OK|MB_ICONWARNING) ;
      }
    }
  UpdateWindow (hwnd) ;
 
  InitCommonControls() ;

  while (GetMessage (&msg, NULL, 0, 0))
    {
    TranslateMessage (&msg) ;
    DispatchMessage (&msg) ;
    }
  return msg.wParam ;
  }

/*-----------------------------------------
   WndProc: Main window callback procedure    
   Version date: 02.12.11
-----------------------------------------*/
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  static char     *szChildClass[] = { "Plan", "Sail", "Map", "Stern" } ;
  static           WNDPROC ChildProc[] = { PlanWndProc, SailWndProc, MapWndProc, SternWndProc } ;
  char            *cButtonText[] = { "Zoom in", "Zoom out", "Increase sail",
                   "Decrease sail", "Helmsman", "Yardsmen", "?", "?", "Pause" } ;
  char            *cCheckText[] = { NULL, NULL, NULL } ;
  static char      szPropsAppName[] = "Props" ;
  WNDCLASSEX       wndclass ;
  HMENU            hMenu ;
  HDC              hdc ;
  PAINTSTRUCT      ps ;
  static int       xClient, yClient, xScrollMax, yScrollMax ;
  int              iI, iID, iSign , iSlider ;
  int              x, cx ;
  int              y, cy ;
  RECT             rWindow ;
  char             sCaption[40];
   
  switch (iMsg)
    {
    case WM_CREATE :
      wndclass.cbSize        = sizeof (wndclass) ;
      wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
      wndclass.cbClsExtra    = 0 ;
      wndclass.cbWndExtra    = 0 ;
      wndclass.hInstance     = hInstance ;
      wndclass.hIcon         = NULL ;
      wndclass.hCursor       = LoadCursor (NULL, IDC_CROSS) ;
      wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
      wndclass.lpszMenuName  = NULL ;
      wndclass.hIconSm       = NULL ;
             
      //create status bar (ID = 19)
      hwndStatus = CreateStatusWindow (WS_CHILDWINDOW | WS_VISIBLE | CCS_BOTTOM ,
        "HMS Surprise", hwnd, 19) ;

      // create custom child windows for planview, sailplan, map and sternview (ID=0 to 3)
      for (iI = 0 ; iI <= 3 ; iI++)
        {
        wndclass.lpfnWndProc   = ChildProc[iI] ;
        wndclass.lpszClassName = szChildClass[iI] ;
        if (iI == 3)
          wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
        RegisterClassEx (&wndclass) ;
        hwndChild[iI] = CreateWindowEx (WS_EX_CLIENTEDGE, szChildClass[iI], NULL,
          WS_CHILDWINDOW | WS_BORDER | WS_VISIBLE | WS_CLIPSIBLINGS , 
          0, 0, 0, 0, hwnd, (HMENU) iI, hInstance, NULL) ;
        }

      // create bitmap for Planview
      hdc = GetDC (hwndChild[0]) ;
      hBitmap = CreateCompatibleBitmap (hdc, xChildPixels, yChildPixels) ;
      ReleaseDC (hwndChild[0], hdc) ;

      // create scrollbars (ID = 4 to 9)
      for (iI=0 ; iI<= 3 ; iI++)
        {
        hwndYard[iI] = CreateWindow ("scrollbar", NULL,
          WS_CHILDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | SBS_HORZ, 0, 0, 0, 0,
          hwnd, (HMENU) (iI+4) , hInstance, NULL) ;
        }

      hwndHelm = CreateWindow ("scrollbar", NULL,
        WS_CHILDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | SBS_HORZ , 0, 0, 0, 0,
        hwnd, (HMENU) 8, hInstance, NULL) ;

      hwndJib = CreateWindow ("scrollbar", NULL,
        WS_CHILDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | SBS_HORZ , 0, 0, 0, 0,
        hwnd, (HMENU) 9, hInstance, NULL) ;

      hdc = GetDC (hwnd) ;
      MyFont = (HFONT) EzCreateFont (hdc, "MS Sans Serif", 80, 0, 0, TRUE) ;
      SelectObject (hdc, MyFont) ;
      SelectObject (hdc, GetStockObject (LTGRAY_BRUSH)) ;
      SetBkColor(hdc, RGB (192,192,192)) ;
      ReleaseDC (hwnd, hdc) ;
 
      //create buttons (ID= 10 to 18)
      for (iI = 0 ; iI <= 8 ; iI++)
        {
        hwndButton[iI] = CreateWindow ("button", cButtonText[iI],
          WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, 0,0,0,0,
          hwnd, (HMENU) (iI+10), hInstance, NULL) ;
        SendMessage(hwndButton[iI], WM_SETFONT, (WPARAM) MyFont, 0) ;
        }

      //create check boxes (ID = 20 to 22)
      for (iI = 0 ; iI <= 2 ; iI++)
        {
        hwndCheck[iI] = CreateWindow ("button", cCheckText[iI],
          WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX, 0,0,0,0,
          hwnd, (HMENU) (iI+20), hInstance, NULL) ;
        SendMessage(hwndCheck[iI], WM_SETFONT, (WPARAM) MyFont, 0) ;
        }

      // register window for changing ship/sail dimensions and properties and islands
      wndclass.lpszClassName = szPropsAppName ;
      wndclass.lpfnWndProc   = PropsProc ;
      wndclass.hbrBackground = (HBRUSH) GetStockObject (LTGRAY_BRUSH) ;
      RegisterClassEx (&wndclass) ;

      // create brushes
      iI= 1 << (GetDeviceCaps (hdc, BITSPIXEL) * GetDeviceCaps (hdc, PLANES)) ;
      if (iI < 64000)
        {
        // 16 or 256 colours
        hSeaBrush = CreateSolidBrush (RGB (0,0,255)) ;
        hLandBrush = CreateSolidBrush (RGB (0,255,0)) ;
        hSkyBrush = CreateSolidBrush (RGB (0,255,255)) ;
        hDeckBrush = CreateSolidBrush (RGB (128,64,0)) ;
        hBlownBrush = CreateSolidBrush (RGB (128,0,0)) ;
        }
      else
        {
        hSeaBrush = CreateSolidBrush (RGB (0,56,207)) ;
        hLandBrush = CreateSolidBrush (RGB (0,173,0)) ;
        hSkyBrush = CreateSolidBrush (RGB (0,215,255)) ;
        hDeckBrush = CreateSolidBrush (RGB (153,102,0)) ;
        hBlownBrush = CreateSolidBrush (RGB (142,0,0)) ;
        }

      FileInitialize() ;

      srand(time (NULL)) ;
      fLastTime = (float) clock()/CLOCKS_PER_SEC ;
      LoadDefault() ;
      DerivedShipParameters() ;
      DerivedSailParameters() ;
      CanShadow() ;
      return 0 ;

    case WM_MOUSEMOVE :
      // display default text in status bar
      LoadString (hInstance, IDS_DEFAULT, sMessage,120) ;
      ShowStatusMessage() ;
      return 0 ;

    case WM_COMMAND :
      hMenu=GetMenu(hwnd) ;
      switch (LOWORD (wParam))
        { 
        case IDM_EXIT :
          SendMessage (hwnd, WM_CLOSE, 0, 0) ;
          return 0 ;
        case IDM_NEW : 
          LoadDefault () ;
          DerivedShipParameters () ;
          DerivedSailParameters () ;
          CanShadow () ;
          strcpy (sCaption, "HMS Surprise");
          SetWindowText (hwnd,sCaption) ;
          return 0 ;
        case IDM_OPEN : 
          if (FileOpenDlg (sFileName,sTitleName)) 
            {
            LoadConfiguration () ;
            DerivedShipParameters () ;
            DerivedSailParameters () ;
            CanShadow () ;
            strcpy (sCaption, "HMS Surprise (");
            strncat (sCaption, sTitleName, 30);
            strcat (sCaption, ")");
            SetWindowText (hwnd,sCaption) ;
            return 0 ;
            }
          else
            return 1 ;
        case IDM_SAVE : 
          if (sFileName[0])
            {
            SaveConfiguration() ;
            return 0 ;
            }
        case IDM_SAVEAS : 
          if (FileSaveDlg (sFileName,sTitleName)) ;
            {
            SaveConfiguration() ;
            strcpy (sCaption, "HMS Surprise (") ;
            strncat (sCaption, sTitleName, 30) ;
            strcat (sCaption, ")") ;
            SetWindowText (hwnd,sCaption) ;
            }
          return 0 ;
        case IDC_ZOOMIN :
          ZoomIn() ;
          return 0 ;
        case IDC_ZOOMOUT :
          ZoomOut() ;
          return 0 ;
        case IDC_YARDSMEN :
          Yardsmen() ;
          return 0 ;
        case IDC_HELMSMAN :
          iBearing = fHeading ;
          DialogBox (hInstance, "HelmBox", hwnd, HelmDlgProc) ;
          return 0 ;
        case IDC_CHANGEJIB1 :
          JibChange(1) ;
          return 0 ;
        case IDC_CHANGEJIB2 :
          JibChange(2) ;
          return 0 ;
        case IDC_PAUSE :
          Pause() ;
          return 0 ;
        case IDM_CONVENTIONAL :
          bParalax = FALSE ;
          CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_UNCHECKED) ;
          return 0 ;
        case IDM_PERSPECTIVE :
          bParalax = TRUE ;
          CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_UNCHECKED) ;
          CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_CHECKED) ;
          return 0 ;
        case IDM_TRACKSHIP :
          bTracking = TRUE ;
          SendMessage (hwndCheck[2], BM_SETCHECK, 1, 0);
          CheckMenuItem(hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
          return 0 ;
        case IDM_DONTTRACKSHIP :
          bTracking = FALSE ;
          SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0);
          CheckMenuItem(hMenu, IDM_DONTTRACKSHIP, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_TRACKSHIP, MF_UNCHECKED) ;
          return 0 ;
        case IDC_TRACKSHIP :
          bTracking = (int) SendMessage (hwndCheck[2], BM_GETCHECK, 0, 0) ;
          if (bTracking)
            {
            CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
            CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
            }
          else
            {
            CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_UNCHECKED) ;
            CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_CHECKED) ;
            }
          bMapMustBePainted = TRUE ;
          return 0 ;
        case IDC_INCRSAIL :
          IncrSail() ;
          return 0 ;
        case IDC_DECRSAIL :
          DecrSail() ;
          return 0 ;
        case IDM_CLEARTRACK :
          iMapCounter = -1 ;
          bMaxTrack = FALSE ;
          bMapMustBePainted = TRUE ;
          return 0 ;
        case IDM_ROYALYARDS :
        case IDM_TOPGALYARDS :
        case IDM_TOPSAILYARDS :
        case IDM_LOWERYARDS :
          CheckMenuItem(hMenu, iYardLevelShown+310, MF_UNCHECKED) ;
          iYardLevelShown = LOWORD (wParam)-310 ;  
          CheckMenuItem(hMenu, iYardLevelShown+310, MF_CHECKED) ;
          bPlanViewIsChanged = TRUE ;
          SetUpDisplay() ; 
          return 0 ;
        case IDM_SLIPSTREAM :
          CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_UNCHECKED) ;
          bShowSlipStream = TRUE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_NOSLIPSTREAM :
          CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_UNCHECKED) ;
          bShowSlipStream = FALSE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_YARDANGLE :
        case IDM_EXPOSEDPERCENTAGE :
        case IDM_EFFECTIVEAREA :
        case IDM_USEFULPERCENTAGE :
        case IDM_USEFULFORCE :
        case IDM_FORWARDTHRUST :
        case IDM_LEEWARDTHRUST :
        case IDM_ROTATIONALTHRUST :    
        case IDM_HEELINGTHRUST :
          CheckMenuItem(hMenu, iDisplayParam+321, MF_UNCHECKED) ;
          iDisplayParam = LOWORD (wParam)-321 ;
          CheckMenuItem(hMenu, iDisplayParam+321, MF_CHECKED) ;
          SelectSailParameter() ;
          return 0 ;
        case IDM_SHIPDIMENSIONS :
        case IDM_SHIPDYNAMIC    :
        case IDM_SAILDIMENSIONS :
        case IDM_STAYSAILS      :
        case IDM_ISLANDS        : 
          EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_SHIPDYNAMIC, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_STAYSAILS, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_ISLANDS, MF_GRAYED) ;
          iModifyMenu = LOWORD (wParam) - IDM_SHIPDIMENSIONS;
          hwndProps = CreateWindowEx (WS_EX_TOPMOST, szPropsAppName, NULL, 
            WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME | WS_CLIPCHILDREN | WS_MINIMIZEBOX | 
            WS_POPUP | WS_CLIPCHILDREN ,
            0, 0, 0, 0, NULL, NULL, hInstance, NULL) ;
          ShowWindow (hwndProps,SW_SHOW) ;
          hwndPropOKButton = CreateWindow ("button", "&OK",
            WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, 0,0,0,0,
            hwndProps, (HMENU) IDC_PROP_OK, hInstance, NULL) ;
          hwndPropCancelButton = CreateWindow ("button", "&Cancel",
            WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, 0,0,0,0,
            hwndProps, (HMENU) IDC_PROP_CANCEL, hInstance, NULL) ;
          SendMessage(hwndPropOKButton, WM_SETFONT, (WPARAM) MyFont, 0) ;
          SendMessage(hwndPropCancelButton, WM_SETFONT, (WPARAM) MyFont, 0) ;
          for (iI=0 ; iI<iFields[iModifyMenu] ; iI++)
            {
            hwndPropText[iI] = CreateWindowEx (WS_EX_CLIENTEDGE,"edit", NULL,
              WS_CHILD | WS_VISIBLE |WS_BORDER | ES_RIGHT, 0,0,0,0,
              hwndProps, (HMENU) iI, hInstance, NULL) ;
              SendMessage(hwndPropText[iI], WM_SETFONT, (WPARAM) MyFont, 0) ;
            fnOldEditProc = (WNDPROC) SetWindowLong (hwndPropText[iI], GWL_WNDPROC, (LONG) EditProc) ;
            }
          PropertiesSetUp() ;
          iFocus = 0 ;
          Focusser () ;
          return 0 ;
        case IDM_ABOUT :
          DialogBox (hInstance, "AboutBox", hwnd, AboutDlgProc) ;
          return 0 ;
        case IDM_HELP :
          WinHelp (hwnd, "Surprise.hlp", HELP_CONTENTS, 0) ;
          return 0 ;
        }

    case WM_MENUSELECT :
      return StatusMessage (hwnd, wParam, lParam) ;

    case WM_PAINT :
      hdc = BeginPaint (hwnd, &ps) ;
      EndPaint (hwnd, &ps) ;
      SetUpDisplay() ;
      PlanView() ;
      SailPlan() ;
      return 0 ;

    case WM_SIZE :
      // position window 
      xClient = LOWORD (lParam) ;
      yClient = HIWORD (lParam) ;
      xScrollMax= max (xMax - xClient,0) ;
      yScrollMax= max (yMax + 16 - yClient,0) ;
      xScrollPos=0 ;
      yScrollPos=0 ;
      hdc = GetDC (hwnd) ;
      SetWindowOrgEx (hdc, 0, 0, NULL) ;
      ReleaseDC (hwnd, hdc) ;
      SetScrollRange (hwnd, SB_HORZ, 0, xScrollMax, FALSE) ;
      SetScrollPos   (hwnd, SB_HORZ, xScrollPos, TRUE) ;
      SetScrollRange (hwnd, SB_VERT, 0, yScrollMax, FALSE) ;
      SetScrollPos   (hwnd, SB_VERT, yScrollPos, TRUE) ;
      // status bar
      cy = rWindow.bottom - rWindow.top ;
      x = 0 ;
      y = yClient - cy ;
      cx = xClient ;
      MoveWindow (hwndStatus, x, y, cx, cy, TRUE) ;
      return 0 ;

    case WM_HSCROLL :
      iID = GetWindowLong ((HWND) lParam, GWL_ID) ;
      iSlider = (int) iID-4 ;
      if (lParam == 0)
        iSlider = 99 ;  
      switch (iSlider)
        {
        case 0 :  
        case 1 :  
        case 2 :  
          // brace yards
          if (bYardsmen)
            break ;
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-1 ;
              break ;
            case SB_LINEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+1 ;
              break ;
            case SB_PAGEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-10 ;
              break ;
            case SB_PAGEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+10 ;
              break ;
            case SB_THUMBPOSITION :
              iNewYardAngle[iSlider]=(int) HIWORD(wParam) ;
              if (iNewYardAngle[iSlider] > 360)
                iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-65536 ;
              break ;
            }
          SetScrollPos (hwndYard[iSlider], SB_CTL, iNewYardAngle[iSlider], TRUE) ;
          YardAdjust(iSlider) ;
          break ;
        case 3 :  
          // brace spanker boom
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+1 ;
              break ;
            case SB_LINEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-1 ;
              break ;
            case SB_PAGEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+10 ;
              break ;
            case SB_PAGEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-10 ;
              break ;
            case SB_THUMBPOSITION :
              iNewYardAngle[iSlider]=-(int) HIWORD(wParam) ;
              if (iNewYardAngle[iSlider] < -360)
                iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+65536 ;
              break ;
            }
          SetScrollPos (hwndYard[iSlider], SB_CTL, -iNewYardAngle[iSlider], TRUE) ;
          YardAdjust(iSlider) ;
          break ;
        case 4 :  
          // helm
          if (bHelmsman && iSlider !=3)
            break ;
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              iNewHelmAngle=iNewHelmAngle-1 ;
              break ;
            case SB_LINEDOWN :
              iNewHelmAngle=iNewHelmAngle+1 ;
              break ;
            case SB_PAGEUP :
              iNewHelmAngle=iNewHelmAngle-10 ;
              break ;
            case SB_PAGEDOWN :
              iNewHelmAngle=iNewHelmAngle+10 ;
              break ;
            case SB_THUMBPOSITION :
              iNewHelmAngle=(int) HIWORD(wParam) ;
              if (iNewHelmAngle > 360)
                iNewHelmAngle=iNewHelmAngle-65536 ;
              break ;
            default :
              break ;
            } 
          SetScrollPos (hwndHelm, SB_CTL, iNewHelmAngle, TRUE) ;
          HelmAdjust() ;
          break ;
        case 5 :  
          // jib
          iSign = Sgn (fSailAngle[12]) ;
          if (iSign == 0)
            iSign = Sgn (fAppWindAngle) ;
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              if ((int) fabs(fSailAngle[12]) > 0)
                fSailAngle[12]=fSailAngle[12] - (float) iSign ;
              break ;
            case SB_LINEDOWN :
              if ((int) fabs(fSailAngle[12]) < iMaxJibAngle)
                fSailAngle[12]=fSailAngle[12] + (float) iSign ;
              break ;
            case SB_PAGEUP :
              if ((int) fabs(fSailAngle[12]) > 9)
                fSailAngle[12]=fSailAngle[12] - 10 * (float) iSign ;
              break ;
            case SB_PAGEDOWN :
              if ((int) fabs(fSailAngle[12]) < (iMaxJibAngle-9))
                fSailAngle[12]=fSailAngle[12] + 10 * (float) iSign ;
              break ;
            case SB_THUMBPOSITION :
              fSailAngle[12]= iSign * (float) HIWORD(wParam) ;
              break ;
            default :
              break ;
            } 
          ShowActualJibAngle () ;
          break ;
        case 99 :  
          // window scrollbar
          switch (LOWORD(wParam))
            {
            case SB_LINEUP :
              xScrollPos = xScrollPos - 1 ;
              break ;
            case SB_LINEDOWN :
              xScrollPos = xScrollPos + 1 ;
              break ;
            case SB_PAGEUP :
              xScrollPos = xScrollPos - 10 ;
              break ;
            case SB_PAGEDOWN :
              xScrollPos = xScrollPos + 10 ;
              break ;
            case SB_THUMBPOSITION :
              xScrollPos = HIWORD(wParam) ;
              break ;
            default:
              break ;
            }
          if (xScrollPos < 0)
            xScrollPos = 0 ;
          if (xScrollPos > (xMax-xClient))
            xScrollPos = xMax-xClient ;
          SetScrollPos (hwnd, SB_HORZ, xScrollPos, TRUE) ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
        } 
      return 0 ;

    case WM_VSCROLL :
      switch (LOWORD(wParam))
        {
        case SB_LINEUP :
          yScrollPos = yScrollPos - 1 ;
          break ;
        case SB_LINEDOWN :
          yScrollPos = yScrollPos + 1 ;
          break ;
        case SB_PAGEUP :
          yScrollPos = yScrollPos - 10 ;
          break ;
        case SB_PAGEDOWN :
          yScrollPos = yScrollPos + 10 ;
          break ;
        case SB_THUMBPOSITION :
          yScrollPos = HIWORD(wParam) ;
          break ;
        default:
          break ;
        }
      if (yScrollPos < 0)
        yScrollPos = 0 ;
      if (yScrollPos > (yMax+16-yClient))
        yScrollPos = yMax+16-yClient ;
      SetScrollPos (hwnd, SB_VERT, yScrollPos, TRUE) ;
      InvalidateRect (hwnd, NULL, TRUE) ;
      return 0 ;

    case WM_DESTROY :
      KillTimer (hwnd, 1) ;
      DeleteObject (hSeaBrush);  
      DeleteObject (hSkyBrush);  
      DeleteObject (hLandBrush);  
      DeleteObject (hDeckBrush);  
      DeleteObject (hBitmap) ;
      hdc = GetDC (hwnd) ;
      DeleteObject (SelectObject(hdc,GetStockObject(SYSTEM_FONT)));  
      ReleaseDC (hwnd, hdc) ;
      PostQuitMessage (0) ;
      return 0 ;
    }
  return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   AboutDlgProc: About Dialog box callback procedure    
   Version date: 99.06.20
-----------------------------------------*/
BOOL CALLBACK AboutDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  switch (iMsg)
    {
    case WM_INITDIALOG :
      return TRUE ;
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDOK :
        case IDCANCEL :
          EndDialog (hDlg, 0) ;
          SetUpDisplay () ;
          return TRUE ;
        }
      break ;
    }
  return FALSE ;
  }

/*-----------------------------------------
   EditProc: configuration input subclass callback procedure    
   Version date: 00.07.16
-----------------------------------------*/

LRESULT CALLBACK EditProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  int    iI ;

  switch (iMsg)
    {
    case WM_KEYDOWN :
      for (iI=0 ; iI<iFields[iModifyMenu] ; iI++)
        if (GetFocus()==hwndPropText[iI])
          iFocus=iI;
      if (wParam == VK_TAB || wParam == VK_RETURN)
        {
        PropertiesChange() ;
        GetKeyState (VK_SHIFT) < 0 ? iFocus-- : iFocus++ ;
        Focusser() ;
        }
      if (wParam == VK_DOWN)
        {
        PropertiesChange() ;
        iFocus++ ;
        Focusser() ;
        }
      if (wParam == VK_UP)
        {
        PropertiesChange() ;
        iFocus-- ;
        Focusser() ;
        }
      break ;
    case WM_LBUTTONDOWN :
      if (iModifyMenu==4)
        bMapMustBePainted=TRUE ;
      PropertiesChange() ;
    case WM_SYSCHAR :
      if (wParam==99)
        PropertiesCancel() ; //alt-C
      if (wParam==111)
        {
        PropertiesChange() ;
        PropertiesAccept() ; //alt-O
        }
    }

  return CallWindowProc (fnOldEditProc, hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   HelmDlgProc: Heading dialog box callback procedure    
   Version date: 99.08.22
-----------------------------------------*/
BOOL CALLBACK HelmDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  float fAngle ;
  int   iTemp ;

  switch (iMsg)
    {
    case WM_INITDIALOG :
      iTackStatus = 0 ;
      SetDlgItemInt (hDlg, IDD_HEADING, iBearing, FALSE) ;
      return TRUE ;
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDD_OK :
          fRequiredHeading = (float) GetDlgItemInt (hDlg, IDD_HEADING, NULL, TRUE) ;
          if ((fRequiredHeading > 360) || (fRequiredHeading< 0))
            {
            fRequiredHeading = iBearing ;
            EndDialog (hDlg, 0) ;
            strcpy(sHelmInstruction, "(await orders)"); 
            SetUpDisplay () ;
            sprintf(sMessage, "Helmsman: await orders"); 
            ShowMessage () ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal heading","Automatic helmsman", MB_OK|MB_ICONERROR) ;
            bHelmsman = FALSE ;
	    return TRUE ;
            }
          EndDialog (hDlg, 0) ;
          bHelmsman = TRUE ;
          bEstablished = FALSE ;
          sprintf(sHelmInstruction, "steer %3.0f    ", fRequiredHeading); 
          SetUpDisplay () ;
          fAngle = fRequiredHeading - (fWindDirection + 180.0) ;
          if (fAngle < -180.0)
            fAngle = fAngle + 360.0 ;
          if (fAngle > 180.0)
            fAngle = fAngle - 360.0 ;
          if (fabs(fAngle) <= 105.0)
            {
            sprintf(sMessage, "Helmsman: steer %3.0f ", fRequiredHeading); 
            ShowMessage () ;
            }
          else
            {
            sprintf(sMessage, "Warning: heading of %3.0f ", fRequiredHeading); 
            ShowMessage () ;
            strcpy(sMessage, "may be too close to wind"); 
            ShowMessage () ;
            }
	  return TRUE ;
        case IDD_CANCEL :
          EndDialog (hDlg, 0) ;
          bHelmsman = FALSE ;
          strcpy(sHelmInstruction, "(await orders)"); 
          SetUpDisplay () ;
          sprintf(sMessage, "Helmsman: await orders"); 
          ShowMessage () ;
          SetUpDisplay () ;
        case IDD_DECR_HEADING :
          iTemp = GetDlgItemInt(hDlg, IDD_HEADING, NULL, TRUE)-1 ;
          if (iTemp<0)
            iTemp = 359 ;
          SetDlgItemInt (hDlg, IDD_HEADING, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_HEADING :
          iTemp = GetDlgItemInt(hDlg, IDD_HEADING, NULL, TRUE)+1 ;
          if (iTemp>360)
            iTemp = 1 ;
          SetDlgItemInt (hDlg, IDD_HEADING, iTemp, FALSE) ;
	  return TRUE ;
        }
     break ;
     }
   return FALSE ;
 }

/*-----------------------------------------
   MapWndProc: Map window callback procedure    
   Version date: 99.10.17
-----------------------------------------*/
LRESULT CALLBACK MapWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  int   xClick ;
  int   yClick ;
  char  sMes[70] ;
  int   iSector ;

  switch (iMsg)
    {
    case WM_MOUSEMOVE :
      // display helpful text in status bar
      xClick = LOWORD (lParam) ;
      yClick = HIWORD (lParam) ;
      iSector = 0 ;
      if (yClick > iMapPixels/3)
        iSector = 3 ;
      if (yClick > 2 * iMapPixels/3)
        iSector = 6 ;
      if (xClick > iMapPixels/3)
        iSector = iSector+1 ;
      if (xClick > 2 * iMapPixels/3)
        iSector = iSector+1 ;
      LoadString (hInstance, IDS_MAP+iSector, sMessage,120) ;
      if (iModifyMenu == 4)
        {
        fNewLong = fLeftEdge + xClick*fMapWidth/iMapPixels ;
        fNewLat =  fTopEdge - yClick*fMapWidth/iMapPixels ;
        sprintf(sMes, "; right-click to place island at N Lat = %5.3f E Long = %5.3f", fNewLat, fNewLong) ;
        strcat(sMessage,sMes) ;
        }
      ShowStatusMessage() ;
      return 0 ;
    case WM_LBUTTONDOWN :
      xClick=LOWORD (lParam) ;
      yClick=HIWORD (lParam) ;
      if (xClick < iMapPixels/3)
        {
        fLeftEdge = fLeftEdge - fMapRadius ;
        fRightEdge = fRightEdge - fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (xClick > 2*iMapPixels/3)
        {
        fLeftEdge = fLeftEdge + fMapRadius ;
        fRightEdge = fRightEdge + fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (yClick < iMapPixels/3)
        {
        fBottomEdge = fBottomEdge + fMapRadius ;
        fTopEdge = fTopEdge + fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (yClick > 2*iMapPixels/3)
        {
        fBottomEdge = fBottomEdge - fMapRadius ;
        fTopEdge = fTopEdge - fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (iModifyMenu == 4)
        SetFocus(hwndPropText[(iFocus/3)*3]) ; 
      return 0 ;  
    case WM_RBUTTONDOWN :
      if (iModifyMenu == 4)
        {
        // move selected island to this position
        xClick=LOWORD (lParam) ;
        yClick=HIWORD (lParam) ;
        fIsleLat[iFocus/3] = fNewLat ;
        fIsleLong[iFocus/3] = fNewLong ;
        sprintf(sMessage, "%1.4f", fIsleLat[iFocus/3]) ;
        SetWindowText (hwndPropText[(iFocus/3)*3],sMessage) ; 
        sprintf(sMessage, "%1.4f", fIsleLong[iFocus/3]) ;
        SetWindowText (hwndPropText[1+(iFocus/3)*3],sMessage) ; 
        SetFocus(hwndPropText[(iFocus/3)*3]) ; 
        bMapMustBePainted = TRUE ;
        }  
       return 0 ;
    }
  return DefWindowProc(hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   PropsProc: Properties adjustment callback procedure    
   Version date: 99.10.17
-----------------------------------------*/
LRESULT CALLBACK PropsProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  HDC          hdc ;
  PAINTSTRUCT  ps ;

  switch (iMsg)     
    {
    case WM_PAINT :
      hdc = BeginPaint (hwndProps, &ps) ;
      EndPaint (hwndProps, &ps) ;
      PropertiesPaint() ;
      return 0;  
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDC_PROP_OK :
          PropertiesChange() ;
          PropertiesAccept () ;
          return 0 ;
        case IDC_PROP_CANCEL :
          PropertiesCancel () ;
          return 0 ;
        }
    case WM_MOUSEMOVE :
      // display default text in status bar
      LoadString (hInstance, IDS_DEFAULT, sMessage,120) ;
      ShowStatusMessage() ;
      return 0 ;
    }
    return DefWindowProc(hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   PlanWndProc: Planview callback procedure    
   Version date: 99.10.03
-----------------------------------------*/

LRESULT CALLBACK PlanWndProc (HWND hwnd2, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  float  fTemp ;
  HDC    hdc ;
  int    xPos, yPos ;

  switch (iMsg) 
    {
    case WM_LBUTTONDOWN :
      DialogBox (hInstance, "WindBox", hwnd, WindDlgProc) ;
      return 0 ;
    case WM_RBUTTONDOWN :
      if (iBearing > -1)
        DialogBox (hInstance, "HelmBox", hwnd, HelmDlgProc) ;
      return 0 ;
    case WM_MOUSEMOVE :
      xPos = -((xChildPixels/2.) - LOWORD(lParam)) ;  // horizontal position of cursor
      yPos = HIWORD(lParam);  // vertical position of cursor 
      if (yPos < (yChildPixels/2))
        yPos = -(yPos-(yChildPixels/2)) ;
      else
        yPos = (yChildPixels/2)-yPos ;
      fTemp = ((float) yPos) /xPos ;
      iBearing = (atan(fTemp) / 0.017456) ;
      if (xPos >= 0 )
        iBearing = 90 - iBearing ;
      if (xPos < 0 )
        iBearing = 270 - iBearing ;
      if (xPos == 0 && yPos >0)
        iBearing = 0 ;
      if (xPos == 0 && yPos <0)
        iBearing = 180 ;
      hdc = GetDC (hwnd) ;
      SelectObject(hdc,GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xBearing+2, yBearing, xBearing - 25, yBearing + 12) ;
      if (sqrt ((xPos*xPos)+(yPos*yPos)) < yChildPixels/4)
        { 
        iBearing = -fWindDirection ;
        // display helpful text in status bar
        LoadString (hInstance, IDS_PLANVIEW_CENT, sMessage,120) ;
        ShowStatusMessage() ;
        }
      else
        { 
        SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
        SetTextAlign (hdc, TA_RIGHT|TA_TOP) ;
        TextOut (hdc, xBearing, yBearing, sMessage, sprintf(sMessage, "%3d ", iBearing)); 
        SetTextAlign (hdc, TA_LEFT|TA_TOP) ;
        fBearingTime = (float) clock()/CLOCKS_PER_SEC ;
        // display helpful text in status bar
        LoadString (hInstance, IDS_PLANVIEW_EDGE, sMessage,120) ;
        ShowStatusMessage() ;
        }
      ReleaseDC (hwnd, hdc) ;
      return 0 ;
    default :
      return DefWindowProc(hwnd2, iMsg, wParam, lParam) ;
    } 
  }

/*-----------------------------------------
   SailWndProc: Sailplan callback procedure    
   Version date: 99.09.26
-----------------------------------------*/
LRESULT CALLBACK SailWndProc (HWND hwnd2, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  HMENU   hMenu ;

  switch (iMsg)
    {
    case WM_MOUSEMOVE :
      // first display helpful text in status bar
      if (bParalax)
        LoadString (hInstance, IDS_SAILPLAN_P, sMessage,120) ;
      else
        LoadString (hInstance, IDS_SAILPLAN_C, sMessage,120) ;
      ShowStatusMessage() ;
      return 0 ;
    case WM_LBUTTONDOWN :
      if (bParalax)
        {
        LoadString (hInstance, IDS_SAILPLAN_C, sMessage,120) ;
        ShowStatusMessage() ;
        bParalax = FALSE ;
        hMenu=GetMenu(hwnd) ;
        CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_CHECKED) ;
        CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_UNCHECKED) ;
        bSailPlanIsChanged = TRUE ;
        return 0 ;
        }
      if (iSailClicks < 9)
        {
        iSailClicks = iSailClicks++ ;
        xSailClick[iSailClicks] = LOWORD (lParam) ;
        ySailClick[iSailClicks] = HIWORD (lParam) ;
        }
      else
        {
        strcpy (sMessage,"Yardsmen overwerked") ;
        ShowMessage () ;
        } 
      return 0;  

    }
  return DefWindowProc(hwnd2, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   StatusMessage: Store message for status bar    
   Version date: 99.09.26
-----------------------------------------*/

LRESULT StatusMessage (HWND hwnd, WPARAM wParam, LPARAM lParam)
  {
  UINT       fuFlags = (UINT) HIWORD (wParam) ;
  
  LoadString (hInstance, (UINT) LOWORD(wParam), sMessage,120) ;

  if (fuFlags == 0xFFFF)  // menu closed
    LoadString (hInstance, IDS_DEFAULT, sMessage,120) ;

  // display saved status message
  ShowStatusMessage() ;
  return 0 ;
  }

/*-----------------------------------------
   SternWndProc: Stern view callback procedure    
   Version date: 99.06.20
-----------------------------------------*/
LRESULT CALLBACK SternWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  return DefWindowProc(hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   WindDlgProc: Wind velocity/direction dialog box callback procedure    
   Version date: 99.08.21
-----------------------------------------*/
BOOL CALLBACK WindDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  int   iTemp ;

  switch (iMsg)
    {
    case WM_INITDIALOG :
      if (iBearing < 0)
        iBearing= -iBearing ;
      SetDlgItemInt (hDlg, IDD_BEARING, iBearing, FALSE) ;
      SetDlgItemInt (hDlg, IDD_B_RATE, iWindDirIncr, FALSE) ;
      SetDlgItemInt (hDlg, IDD_SPEED, (int) fWindSpeed, FALSE) ;
      SetDlgItemInt (hDlg, IDD_S_RATE, iWindSpeedIncr, FALSE) ;
      return TRUE ;
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDD_OK :
          fNewWindDirection = (float) GetDlgItemInt (hDlg, IDD_BEARING, NULL, TRUE) ;
          if ((fNewWindDirection > 360) || (fNewWindDirection< 0))
            {
            fNewWindDirection = fWindDirection ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind direction","Wind direction", MB_OK|MB_ICONERROR) ;
            }
          iWindDirIncr= GetDlgItemInt (hDlg, IDD_B_RATE, NULL, TRUE) ;
          if ((iWindDirIncr > 256) || (iWindDirIncr< 1))
            {
            iWindDirIncr = 128 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind direction change rate","Wind direction", MB_OK|MB_ICONERROR) ;
            }
          fNewWindSpeed = (float) GetDlgItemInt (hDlg, IDD_SPEED, NULL, TRUE) ;
          if ((fNewWindSpeed > 70) || (fNewWindSpeed< 0))
            {
            fNewWindSpeed = fWindSpeed ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind speed","Wind speed", MB_OK|MB_ICONERROR) ;
            }
          iWindSpeedIncr= GetDlgItemInt (hDlg, IDD_S_RATE, NULL, TRUE) ;
          if ((iWindSpeedIncr > 256) || (iWindSpeedIncr< 1))
            {
            iWindSpeedIncr = 128 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind speed change rate","Wind speed", MB_OK|MB_ICONERROR) ;
            }
          EndDialog (hDlg, 0) ;
          if (fNewWindDirection!=fWindDirection)
            {
            strcpy(sMessage, "Wind direction changing"); 
            ShowMessage () ;
            sprintf(sMessage, "to %3.0f  at %3d /min", fNewWindDirection, iWindDirIncr); 
            ShowMessage () ;
            fDeltaWindDir = iWindDirIncr/120. ;
            bWindDirChange = TRUE ;
            }
          if (fNewWindSpeed!=fWindSpeed)
            {
            strcpy(sMessage, "Wind speed changing"); 
            ShowMessage () ;
            sprintf(sMessage, "to %3.0f kts at %3d kts/min", fNewWindSpeed, iWindSpeedIncr); 
            ShowMessage () ;
            fDeltaWindSpeed = iWindSpeedIncr/120. ;
            bWindSpeedChange = TRUE ;
            }
          SetUpDisplay () ;
	  return TRUE ;
	case IDD_CANCEL :
          EndDialog (hDlg, 0) ;
          strcpy(sMessage, "No wind change"); 
          ShowMessage () ;
          SetUpDisplay () ;
	  return TRUE ;
	case IDD_DECR_BEARING :
          iTemp = GetDlgItemInt(hDlg, IDD_BEARING, NULL, TRUE)-1 ;
          if (iTemp<0)
            iTemp = 359 ;
          SetDlgItemInt (hDlg, IDD_BEARING, iTemp, FALSE) ;
	  return TRUE ;
	case IDD_INCR_BEARING :
          iTemp = GetDlgItemInt(hDlg, IDD_BEARING, NULL, TRUE)+1 ;
          if (iTemp>360)
            iTemp = 1 ;
          SetDlgItemInt (hDlg, IDD_BEARING, iTemp, FALSE) ;
	  return TRUE ;
	case IDD_DECR_B_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_B_RATE, NULL, TRUE)/2 ;
          if (iTemp<1)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 1 ;
            }
          SetDlgItemInt (hDlg, IDD_B_RATE, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_B_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_B_RATE, NULL, TRUE)*2 ;
          if (iTemp>128)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 128 ;
            }
          SetDlgItemInt (hDlg, IDD_B_RATE, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_DECR_SPEED :
          iTemp = GetDlgItemInt(hDlg, IDD_SPEED, NULL, TRUE)-1 ;
          if (iTemp<0)
            {
            iTemp = 0 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            }
          SetDlgItemInt (hDlg, IDD_SPEED, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_SPEED :
          iTemp = GetDlgItemInt(hDlg, IDD_SPEED, NULL, TRUE)+1 ;
          if (iTemp>70)
            {
            iTemp = 70 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            }
          SetDlgItemInt (hDlg, IDD_SPEED, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_DECR_S_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_S_RATE, NULL, TRUE)/2 ;
          if (iTemp<1)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 1 ;
            }
          SetDlgItemInt (hDlg, IDD_S_RATE, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_S_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_S_RATE, NULL, TRUE)*2 ;
          if (iTemp>128)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 128 ;
            }
          SetDlgItemInt (hDlg, IDD_S_RATE, iTemp, FALSE) ;
	  return TRUE ;
        }
    break ;
    }
  return FALSE ;
  }

/*-----------------------------------------
   Timer1Proc: Screen update timer callback procedure    
   Version date: 99.10.23
-----------------------------------------*/
VOID CALLBACK Timer1Proc (HWND hwnd, UINT iMsg, UINT iTimerID, DWORD dwTime)
  {
  if (bPaused)
    return ;
  CycleEvents () ;
  ShowParameters () ;
  if (bSailPlanIsChanged || (bPlanViewIsChanged && bParalax))
    SailPlan() ;
  if (bPlanViewIsChanged)
    PlanView() ;
  Map () ;
  if (bSternViewIsChanged)
    SternView() ;
  return ;
  }

/*-----------------------------------------
   Timer2Proc: Ship state calculation timer callback procedure    
   Version date: 99.10.23
-----------------------------------------*/
VOID CALLBACK Timer2Proc (HWND hwnd, UINT iMsg, UINT iTimerID, DWORD dwTime)
  {
  if (bPaused)
    return ;
  Calculate() ;
  }
