/*-----------------------------------------
   PlanView.C -- Plan view procedures module
   Version date: 99.06.21
  -----------------------------------------*/

#define   PI       3.1415927
#define   DEGTORAD PI/180.0

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "external.h"

// Prototypes:
void  PlanView () ;
void  Slipstream (HDC, int, int, float, float, COLORREF) ;

/*-----------------------------------------
   PlanView: Update plan view    
   Version date: 99.04.19
-----------------------------------------*/

void PlanView() 
  {
  float   fAngle, fSinAngle, fCosAngle ;
  float   fDistance ;
  POINT   pDrawHull[7] ;
  float   fRadians ;
  HDC     hdc, hdcMem;
  int     iI ;
  float   fX, fY ;
  int     iThisYard ;
  int     xLeft, xRight, yTop, yBottom ;
  float   fWindArrowPoint ;
  float   fWindArrowHead ;

  bPlanViewIsChanged = FALSE ; //set to True by events requiring redraw

  hdc = GetDC (hwndChild[0]) ;
  hdcMem = CreateCompatibleDC (hdc) ;
  SetMapMode (hdcMem, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdcMem, 1000, 1000, NULL) ;
  SetViewportExtEx (hdcMem, xChildPixels/2, -yChildPixels/2, NULL) ;
  SetViewportOrgEx (hdcMem, xChildPixels/2, yChildPixels/2, NULL) ;
  SetMapMode (hdc, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdc, 1000, 1000, NULL) ;
  SetViewportExtEx (hdc, xChildPixels/2, -yChildPixels/2, NULL) ;
  SetViewportOrgEx (hdc, xChildPixels/2, yChildPixels/2, NULL) ;
  SelectObject (hdcMem, hBitmap) ;
  SelectObject(hdcMem, hSeaBrush) ;
  SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,RGB(0,56,207))) ; //sea blue
  Rectangle (hdcMem, -1000, -1000, 1000, 1000) ;
  DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN))) ; 

  // draw hull
  SelectObject(hdcMem, hDeckBrush) ;
  for (iI=0 ; iI <=6 ; iI++)
    {
    fRadians = fHeading * DEGTORAD ;
    fX=pHull[iI].y*sin(fRadians);
    fY=pHull[iI].y*cos(fRadians);
    fRadians = (fHeading+90) * DEGTORAD ;
    pDrawHull[iI].y=fY+(pHull[iI].x*cos(fRadians)) ;
    pDrawHull[iI].x=fX+(pHull[iI].x*sin(fRadians)) ;
    }
  BeginPath(hdcMem) ;
  PolyBezier(hdcMem,pDrawHull,7);
  CloseFigure(hdcMem) ;
  EndPath(hdcMem) ;
  StrokeAndFillPath(hdcMem) ;

  SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(255,255,0))) ; //yellow
  Ellipse (hdcMem, -10, 10, 10, -10);
  SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;

  //draw foremast yard
  fRadians = fHeading * DEGTORAD ;
  fX = fMastDistance[0] * sin(fRadians) ;
  fY = fMastDistance[0] * cos(fRadians) ;
  xPos[0] = (int) (fX*fScale) ;
  yPos[0] = (int) (fY*fScale) ;
  fDistance = fHeadWidth[iYardLevelShown] / 2. ;
  fRadians = (fHeading + 90.0 + fSailAngle[iYardLevelShown]) * DEGTORAD ;
  xLeft = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
  yTop = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
  xPos[1] = xLeft ;
  yPos[1] = yTop ;
  fRadians = fRadians - PI ;
  if (fRadians < 0)
    fRadians = fRadians + (2 * PI) ;
  xRight = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
  yBottom = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
  xPos[2] = xRight ;
  yPos[2] = yBottom ;
  MoveToEx (hdcMem, xLeft, yTop, NULL) ;
  LineTo (hdcMem, xRight, yBottom) ;
  Ellipse (hdcMem, xPos[0]-20, yPos[0]+20, xPos[0]+20, yPos[0]-20); 

  //draw mainmast yard
  fRadians = fHeading * DEGTORAD ;
  fX = fMastDistance[1] * sin(fRadians) ;
  fY = fMastDistance[1] * cos(fRadians) ;
  xPos[3] = (int) (fX*fScale) ;
  yPos[3] = (int) (fY*fScale) ;
  fDistance = fHeadWidth[iYardLevelShown+4] / 2. ;
  fRadians = (fHeading + 90.0 + fSailAngle[iYardLevelShown+4]) * DEGTORAD ;
  xLeft = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
  yTop = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
  xPos[4] = xLeft ;
  yPos[4] = yTop ;
  fRadians = fRadians - PI ;
  if (fRadians < 0)
    fRadians = fRadians + (2 * PI) ;
  xRight = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
  yBottom = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
  xPos[5] = xRight ;
  yPos[5] = yBottom ;
  MoveToEx (hdcMem, xLeft, yTop, NULL) ;
  LineTo (hdcMem, xRight, yBottom) ;
  Ellipse (hdcMem, xPos[3]-20, yPos[3]+20, xPos[3]+20, yPos[3]-20); 

  // draw crossjack or mizzenyard
  fRadians = fHeading * DEGTORAD ;
  fX = fMastDistance[2] * sin(fRadians) ;
  fY = fMastDistance[2] * cos(fRadians) ;
  xPos[6] = (int) (fX*fScale) ;
  yPos[6] = (int) (fY*fScale) ;
  iThisYard = iYardLevelShown + 8 ;
  if (iYardLevelShown == 3)
    iThisYard = 10 ;
  fDistance = fHeadWidth[iThisYard] / 2. ;
  fRadians = (fHeading + 90. + fSailAngle[iThisYard]) * DEGTORAD ;
  xLeft = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
  yTop = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
  xPos[7] = xLeft ;
  yPos[7] = yTop ;
  fRadians = fRadians - PI ;
  if (fRadians < 0)
    fRadians = fRadians + (2 * PI) ;
  xRight = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
  yBottom = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
  xPos[8] = xRight ;
  yPos[8] = yBottom ;
  MoveToEx (hdcMem, xLeft, yTop, NULL) ;
  LineTo (hdcMem, xRight, yBottom) ;

  // draw mizzenboom
  if (iYardLevelShown == 3)
    {
    fRadians = (fHeading + fSailAngle[11] - 180) * DEGTORAD ;
    if (fRadians > 2 * PI)
      fRadians = fRadians - 2 * PI ;
    xLeft = (int) ((fX * sin(fRadians))*fScale) ;
    yTop = (int) ((fY * cos(fRadians))*fScale) ;
    xRight = (int) ((fX + fFootWidth[11] * sin(fRadians))*fScale) ;
    yBottom = (int) ((fY + fFootWidth[11] * cos(fRadians))*fScale) ;
    xPos[9] = xRight ;
    yPos[9] = yBottom ;
    MoveToEx (hdcMem, xPos[6], yPos[6], NULL) ;
    LineTo (hdcMem, xRight, yBottom) ;
    }
  Ellipse (hdcMem, xPos[6]-20, yPos[6]+20, xPos[6]+20, yPos[6]-20); 

  // draw rudder
  fRadians = fHeading * DEGTORAD ;
  yTop = (int) (((-fShipLength / 2.) * cos(fRadians))*fScale) ;
  xLeft = (int) (((-fShipLength / 2.) * sin(fRadians))*fScale) ;
  fRadians = ((fHeading - 180.) + (float) iHelmAngle) * DEGTORAD ;
  if (fRadians > 2 * PI)
    fRadians = fRadians - (2 * PI) ;
  yBottom = yTop + (int) (7. * cos(fRadians) * fScale) ;
  xRight = xLeft + (int) (7. * sin(fRadians) * fScale) ;
  MoveToEx (hdcMem, xLeft, yTop, NULL) ;
  DeleteObject(SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(0,215,255)))) ; //light blue
  LineTo (hdcMem, xRight, yBottom) ;
  DeleteObject(SelectObject(hdcMem,GetStockObject(WHITE_PEN))) ;

  // draw jib (if not furled or blown out)
  if (iSailStatus[12] < 4)
    {
    fRadians = fHeading * DEGTORAD ;
    fDistance = fMastDistance[3] ;
    fY = fDistance * cos(fRadians) ;
    fX = fDistance * sin(fRadians) ;
    fRadians = (fHeading + fSailAngle[12] - 180.) * DEGTORAD ;
    yBottom = (int) ((fY + fFootWidth[12] * cos(fRadians))*fScale) ;
    xRight = (int) ((fX + fFootWidth[12] * sin(fRadians))*fScale) ;
    xPos[10] = (int) (fX*fScale) ;
    yPos[10] = (int) (fY*fScale) ;
    xPos[11] = xRight ;
    yPos[11] = yBottom ;
    MoveToEx (hdcMem, xPos[10], yPos[10], NULL) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    }

  if (fWindSpeed > 0.01)
    {
    //draw Sail Centre of Effort
    DeleteObject(SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(255,128,128)))) ; //light red
    fRadians = fHeading * DEGTORAD ;
    fDistance = fCOEForeAft ;
    fY = fDistance * cos(fRadians) ;
    fX = fDistance * sin(fRadians) ;
    fRadians = fRadians + (PI / 2.) ;
    fDistance = -fCOELeeward ;
    fY = fY + fDistance * cos(fRadians) ;
    fX = fX + fDistance * sin(fRadians) ;
    xLeft = (int) (fX*fScale) ;
    yTop = (int) (fY*fScale) ;
    MoveToEx (hdcMem, xLeft-30, yTop, NULL) ;
    LineTo (hdcMem, xLeft+40, yTop) ; 
    MoveToEx (hdcMem, xLeft, yTop-35, NULL) ;
    LineTo (hdcMem, xLeft, yTop+40) ; 
  
    // draw actual wind arrow
    fWindArrowPoint = 0.9 * 1000 ;
    fWindArrowHead = 0.05 * 1000 ;
    DeleteObject(SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(39,146,0)))) ; // green
    fRadians = fWindDirection * DEGTORAD ;
    yTop = 1.5 * 1000 * cos(fRadians) ;
    xLeft = 1.5 * 1000 * sin(fRadians) ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    yBottom = fWindArrowPoint * cos(fRadians) ;
    xRight = fWindArrowPoint * sin(fRadians) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    fRadians = (fWindDirection - 140.) * DEGTORAD ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    LineTo (hdcMem, xLeft, yTop) ; 
    fRadians = (fWindDirection + 140.) * DEGTORAD ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    MoveToEx (hdcMem, xRight, yBottom, NULL) ;
    LineTo (hdcMem, xLeft, yTop) ; 

    // draw Apparent wind arrow
    DeleteObject(SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(215,128,128)))) ; //light red
    fAngle = fHeading - 180. - fAppWindAngle ;
    fRadians = (fAngle) * DEGTORAD ;
    yTop = 1.5 * 1000 * cos(fRadians) ;
    xLeft = 1.5 * 1000 * sin(fRadians) ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    yBottom = fWindArrowPoint * cos(fRadians) ;
    xRight = fWindArrowPoint * sin(fRadians) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    fRadians = (fAngle - 140.) * DEGTORAD ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    LineTo (hdcMem, xLeft, yTop) ; 
    fRadians = (fAngle + 140.) * DEGTORAD ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    MoveToEx (hdcMem, xRight, yBottom, NULL) ;
    LineTo (hdcMem, xLeft, yTop) ; 
    DeleteObject(SelectObject(hdcMem,GetStockObject(WHITE_PEN))) ;

    if (bShowSlipStream)
      {
      // dDisplays slipstream of selected yards
      fAngle = 90. + fAppWindAngle - fHeading ; // Convert to PlanView coordinates (0= due east)
      if (fAngle > 360.)
        fAngle = fAngle - 360.0 ;
      fAngle = fAngle * DEGTORAD ;
      fSinAngle = sin(fAngle) ;
      fCosAngle = cos(fAngle) ;

      //calculate slipstream of foresails
      Slipstream(hdcMem, xPos[1], yPos[1], fSinAngle, fCosAngle, RGB(0,0,0)) ; // black
      Slipstream(hdcMem, xPos[2], yPos[2], fSinAngle, fCosAngle, RGB(0,0,0)) ; // black
      //calculate slipstream of mainsails
      Slipstream(hdcMem, xPos[4], yPos[4], fSinAngle, fCosAngle, RGB(255,255,255)) ; // white
      Slipstream(hdcMem, xPos[5], yPos[5], fSinAngle, fCosAngle, RGB(255,255,255)) ; // white

      if (fabs(fAppWindAngle) > 90)
        {
        // wind from foreward semicircle
        if (iSailStatus[12] < 4 )
          {
          // calculate slipstream of jib (if set)
          Slipstream(hdcMem, xPos[10], yPos[10], fSinAngle, fCosAngle, RGB(128,255,128)) ; // light green
          Slipstream(hdcMem, xPos[11], yPos[11], fSinAngle, fCosAngle, RGB(128,255,128)) ; // light green
          }
        }
      else
        {
        // wind from aft semicircle
        if (iYardLevelShown == 3)
          {
          // calculate slipstream of mizzen (gaff)sail
          Slipstream(hdcMem, xPos[6], yPos[6], fSinAngle, fCosAngle, RGB(128,255,128)) ; // light green
          Slipstream(hdcMem, xPos[9], yPos[9], fSinAngle, fCosAngle, RGB(128,255,128)) ; // light green
          }  
        else
          {
          // calculate slipstream of mizzen squaresails
          Slipstream(hdcMem, xPos[7], yPos[7], fSinAngle, fCosAngle, RGB(255,255,255)) ; // white
          Slipstream(hdcMem, xPos[8], yPos[8], fSinAngle, fCosAngle, RGB(255,255,255)) ; // white
          }
        }
      }
    }

  BitBlt (hdc,-1000,-1000,2000,2000,
          hdcMem, -1000,-1000, SRCCOPY) ;
  ReleaseDC (hwndChild[0], hdc) ;
  DeleteDC (hdcMem) ;
  }

/*-----------------------------------------
   Slipstream: Draw sail slipstream    
   Version date: 99.04.17
-----------------------------------------*/

void Slipstream(HDC hdc, int xPos, int yPos, float fSinAngle, float fCosAngle, COLORREF cColor) 
  {
  int   iI ;
  int   xDelta, yDelta ;
  int   xStart, yStart ;

  xDelta = 2000 * fCosAngle ;
  yDelta = 2000 * fSinAngle ;
  xStart = xPos - xDelta ;
  yStart = yPos - yDelta ;
  xDelta = xDelta / 20 ;
  yDelta = yDelta / 20 ;

  for (iI = 0; iI < 40; iI++)
    {
    SetPixel(hdc,xStart,yStart,cColor) ;
    xStart=xStart + xDelta ;
    yStart=yStart + yDelta ;
    } 
  }

