/*-----------------------------------------
   PlanView.C -- Plan view procedures module
   Version date: 03.08.13
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "external.h"

// Prototypes:
void  PlanView () ;
void  Slipstream (HDC, int, int, float, float, COLORREF) ;

/*-----------------------------------------
   PlanView: Update plan view    
   Version date: 03.08.13
-----------------------------------------*/

void PlanView () 
  {
  float   fAngle, fBearing, fCosAngle, fSinAngle ;
  float   fDistance, fTemp ;
  POINT   pDrawHull[7] ;
  float   fRadians ;
  HDC     hdc, hdcMem;
  int     iI ;
  float   fX, fY ;
  int     iThisYard ;
  int     xLeft, xRight, yTop, yBottom ;
  int     xTemp, yTemp ;
  float   fWindArrowPoint ;
  float   fWindArrowHead ;

  bPlanViewIsChanged = FALSE ; //set to True by events requiring redraw

  hdc = GetDC (hwndChild[0]) ;
  hdcMem = CreateCompatibleDC (hdc) ;
  SetMapMode (hdcMem, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdcMem, 1000, 1000, NULL) ;
  SetViewportExtEx (hdcMem, cxChildPixels/2, -cxChildPixels/2, NULL) ;
  SetViewportOrgEx (hdcMem, cxChildPixels/2, cxChildPixels/2, NULL) ;
  SetMapMode (hdc, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdc, 1000, 1000, NULL) ;
  SetViewportExtEx (hdc, cxChildPixels/2, -cxChildPixels/2, NULL) ;
  SetViewportOrgEx (hdc, cxChildPixels/2, cxChildPixels/2, NULL) ;
  SelectObject (hdcMem, hBitmap_0) ;
  SelectObject (hdcMem, hSeaBrush) ;
  Rectangle (hdcMem, -1000, -1000, 1000, 1000) ;

  fBearing = fHeading - fOffsetAngle ;
  if (fBearing > 360)
    fBearing = fBearing - 360.0 ;
  if (fBearing < 0)
    fBearing = fBearing + 360.0 ;

  // draw hull
  SelectObject (hdcMem, hDeckBrush) ;
  for (iI=0 ; iI <=6 ; iI++)
    {
    fRadians = fBearing * fDegToRad ;
    fX=pHull[iI].y*sin(fRadians);
    fY=pHull[iI].y*cos(fRadians);
    fRadians = (fBearing+90) * fDegToRad ;
    pDrawHull[iI].y=fY+(pHull[iI].x*cos(fRadians)) ;
    pDrawHull[iI].x=fX+(pHull[iI].x*sin(fRadians)) ;
    }
  BeginPath (hdcMem) ;
  PolyBezier (hdcMem,pDrawHull,7) ;
  CloseFigure (hdcMem) ;
  EndPath (hdcMem) ;
  StrokeAndFillPath (hdcMem) ;

  //midships datum
  SelectObject (hdcMem, hSkyBluePen) ;
  fRadians = (fBearing + 90.0) * fDegToRad ;
  xLeft = (int) (20 * sin(fRadians)) ;
  yTop = (int) (20 * cos(fRadians)) ;
  MoveToEx (hdcMem, xLeft, yTop, NULL) ;
  LineTo (hdcMem, -xLeft, -yTop) ;

  // Centre of rotation
  SelectObject(hdcMem, hYellowPen) ;
  fRadians = fBearing * fDegToRad ;
  fX = fCORDistance * sin(fRadians) ;
  fY = fCORDistance * cos(fRadians) ;
  xTemp = (int) (fX*fScale) ;
  yTemp = (int) (fY*fScale) ;
  Ellipse (hdcMem, xTemp-10, yTemp+10, xTemp+10, yTemp-10); 

  SelectObject (hdcMem, GetStockObject(BLACK_BRUSH)) ;

  //draw foremast
  if (fMastHeight[0] > fLowestSailFoot[0])
    {
    fRadians = fBearing * fDegToRad ;
    fX = fMastDistance[0] * sin(fRadians) ;
    fY = fMastDistance[0] * cos(fRadians) ;
    xTemp = (int) (fX*fScale) ;
    yTemp = (int) (fY*fScale) ;
    Ellipse (hdcMem, xTemp-20, yTemp+20, xTemp+20, yTemp-20); 
    // draw foremast yard
    switch (iYardLevelShown)
      {
      case 0 :
        iI=0;
        break ;
      case 1 :
        iI=1 ;
        break ;
      case 2 :
        iI=2 ;
        break ;
      case 3 :
        if (bBrig)
          iI=3 ;
        else
          iI=2 ;
        break ;
      case 4 :
        if (bBrig)
          iI=4 ;
        else
          iI=3 ;
      }
    fDistance = fSailHeadWidth[iI] / 2. ;
    fRadians = (fBearing + 90.0 + fSailAngle[iI]) * fDegToRad ;
    xLeft = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
    yTop = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
    xPos[0] = xLeft ;
    yPos[0] = yTop ;
    fRadians = fRadians - fPi ;
    if (fRadians < 0)
      fRadians = fRadians + (2 * fPi) ;
    xRight = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
    yBottom = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
    xPos[1] = xRight ;
    yPos[1] = yBottom ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    LineTo (hdcMem, xRight, yBottom) ;
    }

  //draw mainmast
  if (fMastHeight[1] > fLowestSailFoot[1])
    {
    fRadians = fBearing * fDegToRad ;
    fX = fMastDistance[1] * sin(fRadians) ;
    fY = fMastDistance[1] * cos(fRadians) ;
    xTemp = (int) (fX*fScale) ;
    yTemp = (int) (fY*fScale) ;
    Ellipse (hdcMem, xTemp-20, yTemp+20, xTemp+20, yTemp-20); 
  //draw mainmast yard
    switch (iYardLevelShown)
      {
      case 0 :
        if (bBrig)
          iI=5 ;
        else
          iI=4 ;
        break ;
      case 1 :
        if (bBrig)
          iI=6 ;
        else
          iI=5 ;
        break ;
      case 2 :
        if (bBrig)
          iI=7 ;
        else
          iI=6 ;
        break ;
      case 3 :
        if (bBrig)
          iI=8 ;
        else
          iI=6 ;
        break ;
      case 4 :
        if (bBrig)
          iI=9 ;
        else
          iI=7 ;
      }
    fDistance = fSailHeadWidth[iI] / 2. ;
    fRadians = (fBearing + 90.0 + fSailAngle[iI]) * fDegToRad ;
    xLeft = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
    yTop = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
    xPos[2] = xLeft ;
    yPos[2] = yTop ;
    fRadians = fRadians - fPi ;
    if (fRadians < 0)
      fRadians = fRadians + (2 * fPi) ;
    xRight = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
    yBottom = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
    xPos[3] = xRight ;
    yPos[3] = yBottom ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    LineTo (hdcMem, xRight, yBottom) ;
    }

  //draw mizzenmast
  if (fMastHeight[2] > fLowestSailFoot[2])
    {
    fRadians = fBearing * fDegToRad ;
    fX = fMastDistance[2] * sin(fRadians) ;
    fY = fMastDistance[2] * cos(fRadians) ;
    xTemp = (int) (fX*fScale) ;
    yTemp = (int) (fY*fScale) ;
    Ellipse (hdcMem, xTemp-20, yTemp+20, xTemp+20, yTemp-20); 
    //draw mizzen yard
    switch (iYardLevelShown)
      {
      case 0 :
        iI=8 ;
        break ;
      case 1 :
        iI=9 ;
        break ;
      case 2 :
        iI=10 ;
        break ;
      case 3 :
        iI=10 ;
        break ;
      case 4 :
        iI=10 ;
      }
    fDistance = fSailHeadWidth[iI] / 2. ;
    fRadians = (fBearing + 90.0 + fSailAngle[iI]) * fDegToRad ;
    xLeft = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
    yTop = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
    xPos[4] = xLeft ;
    yPos[4] = yTop ;
    fRadians = fRadians - fPi ;
    if (fRadians < 0)
      fRadians = fRadians + (2 * fPi) ;
    xRight = (int) ((fX + fDistance * sin(fRadians))*fScale) ;
    yBottom = (int) ((fY + fDistance * cos(fRadians))*fScale) ;
    xPos[5] = xRight ;
    yPos[5] = yBottom ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    LineTo (hdcMem, xRight, yBottom) ;
    }

  // draw spanker boom
  fRadians = fBearing * fDegToRad ;
  if (bBrig)
    {
    fX = fMastDistance[1] * sin(fRadians) ;
    fY = fMastDistance[1] * cos(fRadians) ;
    }
  else
    {
    fX = fMastDistance[2] * sin(fRadians) ;
    fY = fMastDistance[2] * cos(fRadians) ;
    }
  xPos[6] = (int) (fX*fScale) ;
  yPos[6] = (int) (fY*fScale) ;
  if (iYardLevelShown == 4)
    {
    fRadians = (fBearing + fSailAngle[iSpanker] - 180) * fDegToRad ;
    if (fRadians > 2 * fPi)
      fRadians = fRadians - 2 * fPi ;
    xLeft = (int) ((fX * sin(fRadians))*fScale) ;
    yTop = (int) ((fY * cos(fRadians))*fScale) ;
    xRight = (int) ((fX + fSailFootWidth[iSpanker] * sin(fRadians))*fScale) ;
    yBottom = (int) ((fY + fSailFootWidth[iSpanker] * cos(fRadians))*fScale) ;
    xPos[7] = xRight ;
    yPos[7] = yBottom ;
    MoveToEx (hdcMem, xPos[6], yPos[6], NULL) ;
    LineTo (hdcMem, xRight, yBottom) ;
    }

  // draw rudder
  fRadians = fBearing * fDegToRad ;
  yTop = (int) (((-fShipLength / 2.) * cos(fRadians))*fScale) ;
  xLeft = (int) (((-fShipLength / 2.) * sin(fRadians))*fScale) ;
  fRadians = ((fBearing - 180.) + (float) iHelmAngle) * fDegToRad ;
  if (fRadians > 2 * fPi)
    fRadians = fRadians - (2 * fPi) ;
  yBottom = yTop + (int) (7. * cos(fRadians) * fScale) ;
  xRight = xLeft + (int) (7. * sin(fRadians) * fScale) ;
  MoveToEx (hdcMem, xLeft, yTop, NULL) ;
  SelectObject(hdcMem, hSkyBluePen) ;
  LineTo (hdcMem, xRight, yBottom) ;

  // draw staysails (if not furled or blown out)
  for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
    {
    if ((iSailStatus[iI] < 4) || (iSailStatus[iI] == 6))
      {
      fRadians = fBearing * fDegToRad ;
      fDistance = fSailTackDistance[iI-(iSpanker+1)] ;
      fY = fDistance * cos(fRadians) ;
      fX = fDistance * sin(fRadians) ;
      fTemp = fSailTackDistance[iI-(iSpanker+1)] - fSailClewDistance[iI-(iSpanker+1)] ;
      if (iSailStatus[iI] < 4)
        fRadians = (fBearing + fSailAngle[iI] - 180.) * fDegToRad ;
      else
        {
        fAngle = fBearing - fAppWindAngle ;
        fRadians = (fAngle) * fDegToRad ;
        }
      yBottom = (int) ((fY + fTemp * cos(fRadians))*fScale) ;
      xRight = (int) ((fX + fTemp * sin(fRadians))*fScale) ;
      xTemp = (int) (fX*fScale) ;
      yTemp = (int) (fY*fScale) ;
      if (iSailStatus[iI] < 4)
        SelectObject(hdcMem,GetStockObject(WHITE_PEN)) ;
      else
        SelectObject(hdcMem, hGrayPen) ;
      MoveToEx (hdcMem, xTemp, yTemp, NULL) ;
      LineTo (hdcMem, xRight, yBottom) ; 
      }
    }

  if (fWindSpeed > 0.01) 
    {
    SelectObject(hdcMem, (HPEN) hPinkPen) ;
    //draw Sail Centre of Effort
    if (fabs(fCOEForeAft) < (fShipLength/2))
      {
      fRadians = fBearing * fDegToRad ;
      fDistance = fCOEForeAft ;
      fY = fDistance * cos(fRadians) ;
      fX = fDistance * sin(fRadians) ;
      fRadians = fRadians + (fPi / 2.) ;
      fDistance = -fCOELeeward ;
      fY = fY + fDistance * cos(fRadians) ;
      fX = fX + fDistance * sin(fRadians) ;
      xLeft = (int) (fX*fScale) ;
      yTop = (int) (fY*fScale) ;
      MoveToEx (hdcMem, xLeft-30, yTop, NULL) ;
      LineTo (hdcMem, xLeft+40, yTop) ; 
      MoveToEx (hdcMem, xLeft, yTop-35, NULL) ;
      LineTo (hdcMem, xLeft, yTop+40) ; 
      }

    // draw actual wind arrow
    fWindArrowPoint = 0.9 * 1000 ;
    fWindArrowHead = 0.05 * 1000 ;
    SelectObject(hdcMem, (HPEN) hThickGreenPen) ;
    fRadians = (fWindDirection-fOffsetAngle) * fDegToRad ;
    yTop = 1.5 * 1000 * cos(fRadians) ;
    xLeft = 1.5 * 1000 * sin(fRadians) ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    yBottom = fWindArrowPoint * cos(fRadians) ;
    xRight = fWindArrowPoint * sin(fRadians) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    fRadians = (fWindDirection -fOffsetAngle - 140.) * fDegToRad ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    LineTo (hdcMem, xLeft, yTop) ; 
    fRadians = (fWindDirection -fOffsetAngle + 140.) * fDegToRad ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    MoveToEx (hdcMem, xRight, yBottom, NULL) ;
    LineTo (hdcMem, xLeft, yTop) ; 

    // draw Apparent wind arrow
    SelectObject(hdcMem, (HPEN) hThickPinkPen) ;
    fAngle = fBearing - 180. - fAppWindAngle ;
    fRadians = (fAngle) * fDegToRad ;
    yTop = 1.5 * 1000 * cos(fRadians) ;
    xLeft = 1.5 * 1000 * sin(fRadians) ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    yBottom = fWindArrowPoint * cos(fRadians) ;
    xRight = fWindArrowPoint * sin(fRadians) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    fRadians = (fAngle - 140.) * fDegToRad ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    LineTo (hdcMem, xLeft, yTop) ; 
    fRadians = (fAngle + 140.) * fDegToRad ;
    yTop = yBottom - fWindArrowHead * cos(fRadians) ;
    xLeft = xRight - fWindArrowHead * sin(fRadians) ;
    MoveToEx (hdcMem, xRight, yBottom, NULL) ;
    LineTo (hdcMem, xLeft, yTop) ; 

    if (bShowSlipStream)
      {
      // dDisplays slipstream of selected yards
      fAngle = 90. + fAppWindAngle - fBearing ; // Convert to PlanView coordinates (0= due east)
      if (fAngle > 360.)
        fAngle = fAngle - 360.0 ;
      fAngle = fAngle * fDegToRad ;
      fSinAngle = sin(fAngle) ;
      fCosAngle = cos(fAngle) ;

      //calculate slipstream of foresails
      if (iSailStatus[iYardLevelShown] < 7)
        {
        Slipstream(hdcMem, xPos[0], yPos[0], fSinAngle, fCosAngle, RGB(128,255,128)) ; // light green
        Slipstream(hdcMem, xPos[1], yPos[1], fSinAngle, fCosAngle, RGB(128,255,128)) ; // light green
        }
      //calculate slipstream of mainsails
      if (iSailStatus[5 + iYardLevelShown] < 7)
        {
        Slipstream(hdcMem, xPos[2], yPos[2], fSinAngle, fCosAngle, RGB(255,255,255)) ; // white
        Slipstream(hdcMem, xPos[3], yPos[3], fSinAngle, fCosAngle, RGB(255,255,255)) ; // white
        }
      if (iYardLevelShown == 4)
        {
        // calculate slipstream of spanker
        Slipstream(hdcMem, xPos[6], yPos[6], fSinAngle, fCosAngle, RGB(0,0,0)) ; // black
        Slipstream(hdcMem, xPos[9], yPos[9], fSinAngle, fCosAngle, RGB(0,0,0)) ; // black
        }
      }
    }

  // draw compass bowl
  if (bCompassRing)
    {
    SelectObject (hdcMem, hGrayPen) ;
    SelectObject (hdcMem, GetStockObject(NULL_BRUSH)) ;
    Ellipse (hdcMem, -950, 950, 950, -950); 
    for (iI=10 ; iI< 360 ; iI=iI+10)
      {
      fRadians = (-fOffsetAngle+iI) * fDegToRad ;
      yTop = 950.0 * cos(fRadians) ;
      xLeft = 950.0 * sin(fRadians) ;
      MoveToEx (hdcMem, xLeft, yTop, NULL) ;
      yTop = 980.0 * cos(fRadians) ;
      xLeft = 980.0 * sin(fRadians) ;
      LineTo (hdcMem, xLeft, yTop) ; 
      }
    SelectObject(hdcMem, (HPEN) hThickGrayPen) ;
    for (iI=90 ; iI <= 270 ; iI=iI+90)
      {
      fRadians = (-fOffsetAngle+iI) * fDegToRad ;
      yTop = 900.0 * cos(fRadians) ;
      xLeft = 900.0 * sin(fRadians) ;
      MoveToEx (hdcMem, xLeft, yTop, NULL) ;
      yTop = 980.0 * cos(fRadians) ;
      xLeft = 980.0 * sin(fRadians) ;
      LineTo (hdcMem, xLeft, yTop) ; 
      }
    fRadians = (-fOffsetAngle) * fDegToRad ;
    yTop = 850.0 * cos(fRadians) ;
    xLeft = 850.0 * sin(fRadians) ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    yTop = 1080.0 * cos(fRadians) ;
    xLeft = 1080.0 * sin(fRadians) ;
    LineTo (hdcMem, xLeft, yTop) ; 
    fRadians = (fRadians - 2.443) ;
    yBottom = yTop + fWindArrowHead * cos(fRadians) ;
    xRight = xLeft + fWindArrowHead * sin(fRadians) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    fRadians = (fRadians + 4.886) ;
    yBottom = yTop + fWindArrowHead * cos(fRadians) ;
    xRight = xLeft + fWindArrowHead * sin(fRadians) ;
    MoveToEx (hdcMem, xLeft, yTop, NULL) ;
    LineTo (hdcMem, xRight, yBottom) ; 
    SelectObject(hdcMem,GetStockObject(WHITE_PEN)) ;
    MoveToEx (hdcMem, 0, 980, NULL) ;
    LineTo (hdcMem, 0, 1020) ; 
    }

  BitBlt (hdc,-1000,-1000,2000,2000,
          hdcMem, -1000,-1000, SRCCOPY) ;
  ReleaseDC (hwndChild[0], hdc) ;
  DeleteDC (hdcMem) ;
  }

/*-----------------------------------------
   Slipstream: Draw sail slipstream    
   Version date: 99.04.17
-----------------------------------------*/

void Slipstream(HDC hdc, int xPos, int yPos, float fSinAngle, float fCosAngle, COLORREF cColor) 
  {
  int   iI ;
  int   xDelta, yDelta ;
  int   xStart, yStart ;

  xDelta = 2000 * fCosAngle ;
  yDelta = 2000 * fSinAngle ;
  xStart = xPos - xDelta ;
  yStart = yPos - yDelta ;
  xDelta = xDelta / 20 ;
  yDelta = yDelta / 20 ;

  for (iI = 0; iI < 40; iI++)
    {
    SetPixel(hdc,xStart,yStart,cColor) ;
    xStart=xStart + xDelta ;
    yStart=yStart + yDelta ;
    } 
  }

