/* --------------------------------------
   Procedure SailPlan
   Version date = 04.03.07
----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "external.h"
#include "menu.h"

// Prototypes:
void  DrawMast (HDC, int, float) ;
void  DrawSail (HDC, char) ;
void  LargerJib () ;
void  NewJib () ;
void  SailPlan () ;
void  ShowBow (HDC, float) ;
void  ShowStern (HDC, float) ;
void  SmallerJib () ;

extern int   Sgn (float) ;
extern void  ShowActualJibAngle () ;
extern void  ShowMessage () ;

/* ------------------------------
     DrawMast
     Version date: 03.03.22
-------------------------------*/

void DrawMast(HDC hdcMem, int iMast, float fFactor)
  {
  SelectObject (hdcMem, (HPEN) hMastPen) ;
  MoveToEx (hdcMem, fMastDistance[iMast]*fScaleSail*fFactor,0, NULL) ;
  LineTo (hdcMem, fMastDistance[iMast]*fScaleSail*fFactor,fMastHeight[iMast]*fScaleSail) ;
  SelectObject (hdcMem,GetStockObject(BLACK_PEN)) ; 
  }
 
/* ------------------------------
     DrawSail: draws a sail on the conventional sail plan view
     Version date: 03.09.03
-------------------------------*/

void DrawSail(HDC hdc, char cSl)
  {
  float   yBandBase, yBandTop ;
  float   fBotLeft, fBotRight ;
  int     iJ ;
  int     iNrBands ;
  float   fOffSetX ;
  int     iSlStatus ;
  float   fSlantLeft ;
  float   fSlantRight ;
  float   fTemp, fTemp2 ;
  float   fTopLeft, fTopRight ;

  iSlStatus = iSailStatus[cSl] ;
  fSlantLeft = (xSailPlanFootLeft[cSl] - xSailPlanHeadLeft[cSl]) / (fSailHead[cSl] - fSailFoot[cSl]) ;
  fSlantRight = (xSailPlanFootRight[cSl] - xSailPlanHeadRight[cSl]) / (fSailHead[cSl] - fSailFoot[cSl]) ;
  iNrBands = iSailMaxReefs[cSl] + 1 ;

  switch (iSailType[cSl])
    {
    case 2:
      fOffSetX = fMastDistance[0] ; // fore sails
      break ;
    case 4:
      fOffSetX = fMastDistance[1] ;  // mainsails
      break ;
    case 6:
      fOffSetX = fMastDistance[2] ;  // mizzen sails
      break ;
    case 1:
    case 3:
      fOffSetX = fSailTackDistance[cSl-(iSpanker+1)] ; // staysails
    }

  yBandTop = ySailPlanHeadLeft[cSl] ;
  for (iJ=1; iJ <= iNrBands ; iJ++)
    {
    if (iJ == iNrBands) 
      yBandBase = ySailPlanFootLeft[cSl] ;
    else
      yBandBase = ySailPlanFootLeft[cSl] + (ySailPlanHeadLeft[cSl] - ySailPlanFootLeft[cSl]) * fSetFraction[iJ] ;
    switch (iSlStatus)
      {
      case 0:
        {
        // Set
        SelectObject(hdc, (HBRUSH) GetStockObject(LTGRAY_BRUSH)) ;
        if (cSl >= iSpanker)
          SelectObject(hdc, (HBRUSH) hForeAftBrush) ;
        SelectObject(hdc, (HPEN) GetStockObject(BLACK_PEN)) ;
        break ;
        }
      case 4:
        // Furled
        {
        SelectObject(hdc, (HBRUSH) GetStockObject(NULL_BRUSH)) ;
        SelectObject(hdc, (HPEN) GetStockObject(WHITE_PEN)) ;
        break ;
        }
      case 5:
        // Blown out
        {
        SelectObject(hdc, (HBRUSH) hBlownBrush) ;
        SelectObject(hdc, (HPEN) GetStockObject(BLACK_PEN)) ;
        break ;
        } 
      case 6:
        // Eased
        {
        SelectObject(hdc, (HBRUSH) hForeAftBrush) ;
        SelectObject(hdc, (HPEN) GetStockObject(WHITE_PEN)) ;
        break ;
        } 
      default :
        // Reefed
        {
        if (iJ < (iSlStatus+1))
          {
          // this band furled
          SelectObject(hdc, (HBRUSH) GetStockObject(NULL_BRUSH)) ;
          SelectObject(hdc, (HPEN) GetStockObject(WHITE_PEN)) ;
          }
        else
          {
          // this band set
          SelectObject(hdc, (HBRUSH) GetStockObject(LTGRAY_BRUSH)) ;
          if (cSl >= iSpanker)
            SelectObject(hdc, (HBRUSH) hForeAftBrush) ;
          SelectObject(hdc, (HPEN) GetStockObject(BLACK_PEN)) ;
          }
        }
      } 
    if (cSl < iSpanker)
      {
      fTopLeft = fOffSetX + xSailPlanHeadLeft[cSl] + (fSlantLeft * (fSailHead[cSl]-yBandTop)) ;
      fTopRight = fOffSetX + xSailPlanHeadRight[cSl] + (fSlantRight * (fSailHead[cSl]-yBandTop)) ;
      fBotRight = fOffSetX + xSailPlanHeadRight[cSl] + (fSlantRight * (fSailHead[cSl]-yBandBase)) ;
      fBotLeft = fOffSetX + xSailPlanHeadLeft[cSl] + (fSlantLeft * (fSailHead[cSl]-yBandBase)) ;
      BeginPath(hdc) ;
      MoveToEx (hdc, fTopLeft*fScaleSail, yBandTop*fScaleSail, NULL) ;
      LineTo (hdc, fTopRight*fScaleSail, yBandTop*fScaleSail) ;
      LineTo (hdc, fBotRight*fScaleSail, yBandBase*fScaleSail) ;
      LineTo (hdc, fBotLeft*fScaleSail, yBandBase*fScaleSail) ;
      CloseFigure(hdc) ;
      EndPath(hdc) ;
      StrokeAndFillPath(hdc) ;
      yBandTop = yBandBase ;
      }
    else
      {
      if (cSl == iSpanker)
        {
        fTopLeft = fOffSetX + xSailPlanHeadLeft[cSl] ;
        fTopRight = fOffSetX + xSailPlanHeadRight[cSl] ;
        fBotRight = fOffSetX + xSailPlanHeadRight[cSl] ;
        fBotLeft = fOffSetX + xSailPlanHeadLeft[cSl] ;
        BeginPath(hdc) ;
        MoveToEx (hdc, fTopLeft*fScaleSail, yBandTop*fScaleSail, NULL) ;
        LineTo (hdc, fTopRight*fScaleSail, (yBandTop+fSpankerPeak)*fScaleSail) ;
        if (iJ == iNrBands)
          LineTo (hdc, fBotRight*fScaleSail, yBandBase*fScaleSail) ;
        else
          LineTo (hdc, fBotRight*fScaleSail, (yBandBase+fSpankerPeak)*fScaleSail) ;
        LineTo (hdc, fBotLeft*fScaleSail, yBandBase*fScaleSail) ;
        CloseFigure(hdc) ;
        EndPath(hdc) ;
        StrokeAndFillPath(hdc) ;
        yBandTop = yBandBase ;
        }
      else
        {
        // jib or staysail
        BeginPath(hdc) ;
        MoveToEx (hdc, (fOffSetX+xSailPlanHeadLeft[cSl])*fScaleSail,ySailPlanHeadLeft[cSl]*fScaleSail,NULL) ;
        LineTo (hdc, (fOffSetX+xSailPlanHeadRight[cSl])*fScaleSail,ySailPlanHeadRight[cSl]*fScaleSail) ;
        LineTo (hdc, (fOffSetX+xSailPlanFootRight[cSl])*fScaleSail,ySailPlanFootRight[cSl]*fScaleSail) ;
        LineTo (hdc, (fOffSetX+xSailPlanFootLeft[cSl])*fScaleSail,ySailPlanFootLeft[cSl]*fScaleSail) ;
        CloseFigure(hdc) ;
        EndPath(hdc) ;
        StrokeAndFillPath(hdc) ;
        }
      }
    }
  }

/*-----------------------------------------
   LargerJib: Sets larger jib on frigate    
   Version date: 03.09.03
-----------------------------------------*/
void LargerJib () 
  {
  HDC   hdc ;
  hdc = GetDC (hwnd) ;

  switch (iJibSet)
    {
    case 1:
    case 2:
      iJibSet=0 ;
      SetWindowText(hwndButton[7], "Storm jib") ;
      SetWindowText(hwndButton[8], "Working jib") ;
      TextOut (hdc, xJib, yMessage+iSpacer, "Large jib:      ",16) ;
      break ;
    case 0:
      iJibSet=1 ;
      SetWindowText(hwndButton[7], "Storm jib") ;
      SetWindowText(hwndButton[8], "Large jib") ;
      TextOut (hdc, xJib, yMessage+iSpacer, "Working jib:",12) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  SetFocus (hwndCheck[1]) ;
  NewJib () ;
  }

/*-----------------------------------------
   NewJib: Recalculate after selection of new jib    
   Version date: 03.09.03
-----------------------------------------*/
void NewJib () 
  {
  fSailClewDistance[0]= fSailClewDistance[iJibSet+1] ;
  fSailClewElevation[0]= fSailClewElevation[iJibSet+1] ;
  fSailTackDistance[0]= fSailTackDistance[iJibSet+1] ;
  fSailTackElevation[0]= fSailTackElevation[iJibSet+1] ;
  fSailHeadDistance[0]= fSailHeadDistance[iJibSet+1] ;
  fSailHeadElevation[0]= fSailHeadElevation[iJibSet+1] ;
  // Recalculate effective head and mean heights of sail
  fSailEffectiveHead[12] = fSailHeadElevation[0] ;
  fSailFoot[12] = fSailTackElevation[0] ;
  fSailMeanHeight[12] = (fSailTackElevation[0]+fSailClewElevation[0]+fSailHeadElevation[0])/3.0;
  fSailSSCentroid[12] = (fSailTackDistance[0]+fSailClewDistance[0]+fSailHeadDistance[0])/3.0;
  xSailPlanHeadLeft[12] = -(fSailTackDistance[0]-fSailHeadDistance[0]) ;
  xSailPlanHeadRight[12] = -(fSailTackDistance[0]-fSailHeadDistance[0]) ;
  xSailPlanFootLeft[12] = -(fSailTackDistance[0]-fSailClewDistance[0]) ;
  xSailPlanFootRight[12] = 0 ;
  ySailPlanHeadLeft[12] = fSailHeadElevation[0] ;
  ySailPlanHeadRight[12] = fSailHeadElevation[0] ;
  ySailPlanFootLeft[12] = fSailClewElevation[0] ;
  ySailPlanFootRight[12] = fSailTackElevation[0] ;
  fSailCorner[12] = fSailClewElevation[0] ;

  bSailPlanIsChanged = TRUE ;
  CanShadow () ;
  bPlanViewIsChanged = TRUE ;
  }

/*-----------------------------------------
   SailPlan: Updates sailplan    
   Version date: 04.03.07
-----------------------------------------*/

void SailPlan () 
  {
  int    iI, iJ ;
  HDC    hdc, hdcMem ;
  float  fFactor ;
  int    iSl ;
  float  fScew ;
  float  fStep ;
  float  fTemp ;
  POINT  pPoint ;
  int    xStart, yStart ;

  if (bPaused)
    return ;

  bSailPlanIsChanged = FALSE ;
  hdc = GetDC (hwndChild[1]) ;
  hdcMem = CreateCompatibleDC (hdc) ;
  fScaleSail = 850.0 / (fHighest+10.0) ;
  if (bBrig)
    {
    iI=1 ;
    iJ=0 ;
    }
  else
    { 
    iI=2 ;
    iJ=1 ;
    }
  fScaleSail = 1000.0/(fabs(fMaxAft) + fMaxFore + 10.) ; 
  fScew = abs(fMaxAft)/(fMaxFore+abs(fMaxAft)); 
  if (bParalax)
    fScew = 0.5 + (fScew - 0.5) * sin((180-fAppWindAngle)*fDegToRad) ;

  xOrgPixels = cxChildPixels * fScew ;
  SetMapMode (hdc, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdc, 1000 * fScew, 850, NULL) ;
  SetViewportExtEx (hdc, xOrgPixels, -0.85*cxChildPixels, NULL) ;
  SetViewportOrgEx (hdc, xOrgPixels, 0.85*cxChildPixels, NULL) ;
  SetMapMode (hdcMem, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdcMem, 1000 * fScew, 850, NULL) ;
  SetViewportExtEx (hdcMem, xOrgPixels, -0.85*cxChildPixels, NULL) ;
  SetViewportOrgEx (hdcMem, xOrgPixels, 0.85*cxChildPixels, NULL) ;
  SelectObject (hdcMem, hBitmap_1) ;
  SelectObject(hdcMem, (HPEN) hSeaBluePen) ;
  SelectObject(hdcMem, hSeaBrush) ;
  Rectangle (hdcMem, -1000, 0, 1000, -150) ;
  SelectObject (hdcMem, (HPEN) hSkyBluePen) ;
  SelectObject (hdcMem, hSkyBrush) ;
  Rectangle (hdcMem, -1000, 0, 1000, 850) ;
  SelectObject(hdcMem,GetStockObject(BLACK_PEN)) ; 

  if (!bParalax)
    {
    // conventional sailplan

    // draw hull
    SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
    fTemp = fScaleSail * fShipLength/2.0 ;
    BeginPath(hdcMem) ;
    MoveToEx (hdcMem, -fTemp, 0., NULL) ;
    LineTo (hdcMem, -fTemp, fQDeckHeight*fScaleSail) ;
    LineTo (hdcMem, fQDeckLength*fScaleSail - fTemp, fQDeckHeight*fScaleSail) ;
    LineTo (hdcMem, fQDeckLength*fScaleSail - fTemp, fGunDeckHeight*fScaleSail) ;
    LineTo (hdcMem, fTemp - fFocleLength*fScaleSail, fGunDeckHeight*fScaleSail) ;
    LineTo (hdcMem, fTemp - fFocleLength*fScaleSail, fQDeckHeight*fScaleSail) ;
    LineTo (hdcMem, fTemp, fQDeckHeight*fScaleSail) ;
    LineTo (hdcMem, fTemp - fTemp*0.2, 0.) ;
    CloseFigure(hdcMem) ;
    EndPath(hdcMem) ;
    StrokeAndFillPath(hdcMem) ;

    // draw masts
    for (iI=0 ; iI<=2 ; iI++)
      {
      if (fMastHeight[iI] > fLowestSailFoot[iI])
        {
        MoveToEx (hdcMem, fMastDistance[iI]*fScaleSail, 0., NULL) ;
        LineTo (hdcMem, fMastDistance[iI]*fScaleSail,fMastHeight[iI]*fScaleSail) ;
        }
      }
    // draw sails
    if (bBrig)
      {
      for (iI = 0 ; iI < iNrSails ; iI++)
        DrawSail(hdcMem, iBrigSailSequence[4][iI]) ;
      }
    else
      { 
      for (iI = 0 ; iI < iNrSails ; iI++)
        DrawSail(hdcMem, iFrigateSailSequence[4][iI]) ;
      }
    }

  else

    {

    // view from direction of wind

    fFactor = sin(fAppWindAngle * fDegToRad) ;

    if (!bBrig) 
      {
      // draw barricade
      if ( fabs(fAppWindAngle) <90 )
        fTemp = ((fShipLength/2.0)-fFocleLength)*fScaleSail * fFactor; // forecastle 
      else 
        fTemp = -((fShipLength/2.0)-fQDeckLength)*fScaleSail * fFactor; // quarterdeck
      SelectObject(hdcMem, hDeckBrush) ;
      fStep=fShipWidth*0.5*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
      BeginPath(hdcMem) ;
      MoveToEx (hdcMem, fTemp-fStep, fGunDeckHeight*fScaleSail, NULL) ;
      LineTo (hdcMem, fTemp-fStep, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp+fStep, fGunDeckHeight*fScaleSail) ;
      CloseFigure(hdcMem) ;
      EndPath(hdcMem) ;
      StrokeAndFillPath(hdcMem) ;
      // draw hull
      fTemp = fShipLength*fScaleSail/2.0 * fFactor;
      SelectObject(hdcMem, GetStockObject (BLACK_BRUSH)) ;
      BeginPath(hdcMem) ;
      MoveToEx (hdcMem, -fTemp, 0., NULL) ;
      LineTo (hdcMem, -fTemp, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fQDeckLength*fScaleSail*fFactor - fTemp, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fQDeckLength*fScaleSail*fFactor - fTemp, fGunDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp - fFocleLength*fScaleSail*fFactor, fGunDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp - fFocleLength*fScaleSail*fFactor, fQDeckHeight*fScaleSail) ;
      fStep=Sgn(fAppWindAngle)*fShipWidth*0.5*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
      LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp+fStep, 0.) ;
      CloseFigure(hdcMem) ;
      EndPath(hdcMem) ;
      StrokeAndFillPath(hdcMem) ;
      }
    else
      {
      // draw hull
      fTemp = fShipLength*fScaleSail/2.0 * fFactor;
      SelectObject(hdcMem, GetStockObject (BLACK_BRUSH)) ;
      BeginPath(hdcMem) ;
      MoveToEx (hdcMem, -fTemp, 0., NULL) ;
      LineTo (hdcMem, -fTemp, fQDeckHeight*fScaleSail) ;
      fStep=Sgn(fAppWindAngle)*fShipWidth*0.5*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
      LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp+fStep, 0.) ;
      CloseFigure(hdcMem) ;
      EndPath(hdcMem) ;
      StrokeAndFillPath(hdcMem) ;
      SelectObject(hdcMem, GetStockObject (WHITE_BRUSH)) ;
      BeginPath(hdcMem) ;
      MoveToEx (hdcMem, fQDeckLength*fScaleSail*fFactor - fTemp, fQDeckHeight*fScaleSail, NULL) ;
      LineTo (hdcMem, fQDeckLength*fScaleSail*fFactor - fTemp, fGunDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp - fFocleLength*fScaleSail*fFactor, fGunDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp - fFocleLength*fScaleSail*fFactor, fQDeckHeight*fScaleSail) ;
      CloseFigure(hdcMem) ;
      EndPath(hdcMem) ;
      StrokeAndFillPath(hdcMem) ;
      // draw superstructure
      if ( fabs(fAppWindAngle) >90 )
        {
        SelectObject(hdcMem, (HBRUSH) GetStockObject(WHITE_BRUSH)) ;
        fTemp = ((fShipLength/2.0)-fFocleLength)*fScaleSail * fFactor; // forecastle 
        fStep=fShipWidth*0.4*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
        BeginPath(hdcMem) ;
        MoveToEx (hdcMem, fTemp-fStep, fGunDeckHeight*fScaleSail, NULL) ;
        LineTo (hdcMem, fTemp-fStep, fQDeckHeight*fScaleSail) ;
        LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fScaleSail) ;
        LineTo (hdcMem, fTemp+fStep, fGunDeckHeight*fScaleSail) ;
        CloseFigure(hdcMem) ;
        EndPath(hdcMem) ;
        StrokeAndFillPath(hdcMem) ;
        }
      }

    // draw masts and and sails

    DrawMast(hdcMem,0,fFactor) ;  // fore mast
    DrawMast(hdcMem,1,fFactor) ;  // main mast
    if (!bBrig)
      DrawMast(hdcMem,2,fFactor) ;  // mizzen mast

    for (iI = 0 ; iI < iNrSails ; iI++)
      {
      if (bBrig) 
        iSl=iBrigSailSequence[iSeq][iI] ;
      else
        iSl=iFrigateSailSequence[iSeq][iI] ;
      switch (iSailType[iSl]) 
        {
        case 1 :  
        case 3 :  
          SelectObject(hdcMem, hStaySailBrush) ;
          break ;
        case 2 :  
          SelectObject(hdcMem, hForeSailBrush) ;
          break ;
        case 4 :  
          SelectObject(hdcMem, hMainSailBrush) ;
          break ;
        case 6 :  
          SelectObject(hdcMem, hForeSailBrush) ;
          break ;
        }

      if ((iSailStatus[iSl]<4) || (iSailStatus[iSl]==6))
        {
        if (iSl<iSpanker)
          {
          BeginPath (hdcMem) ;
          MoveToEx (hdcMem, xSailWindHeadLeft[iSl]*fScaleSail, fSailEffectiveHead[iSl]*fScaleSail, NULL) ;
          LineTo (hdcMem, xSailWindHeadRight[iSl]*fScaleSail, fSailEffectiveHead[iSl]*fScaleSail) ;
          LineTo (hdcMem, xSailWindFootRight[iSl]*fScaleSail, ySailPlanFootRight[iSl]*fScaleSail) ;
          LineTo (hdcMem, xSailWindFootLeft[iSl]*fScaleSail, ySailPlanFootLeft[iSl]*fScaleSail) ;
          CloseFigure (hdcMem) ;
          EndPath (hdcMem) ;
          StrokeAndFillPath (hdcMem) ;
          }
        else 
          {
          if (iSailStatus[iSl] == 6)
            {
            //eased
            MoveToEx (hdcMem, xSailWindHeadLeft[iSl]*fScaleSail, ySailPlanHeadLeft[iSl]*fScaleSail, NULL) ;
            LineTo (hdcMem, xSailWindFootRight[iSl]*fScaleSail, ySailPlanFootRight[iSl]*fScaleSail) ;
            }
          else
            {
            //not eased
            if (iSl == iSpanker)
              {
              BeginPath (hdcMem) ;
              MoveToEx (hdcMem, xSailWindHeadLeft[iSl]*fScaleSail, (fSailEffectiveHead[iSl]-fSpankerPeak)*fScaleSail, NULL) ;
              LineTo (hdcMem, xSailWindHeadRight[iSl]*fScaleSail, fSailEffectiveHead[iSl]*fScaleSail) ;
              LineTo (hdcMem, xSailWindFootRight[iSl]*fScaleSail, ySailPlanFootRight[iSl]*fScaleSail) ;
              LineTo (hdcMem, xSailWindFootLeft[iSl]*fScaleSail, ySailPlanFootLeft[iSl]*fScaleSail) ;
              CloseFigure (hdcMem) ;
              EndPath (hdcMem) ;
              StrokeAndFillPath (hdcMem) ;
              }
            else
              {
              BeginPath (hdcMem) ;
              MoveToEx (hdcMem, xSailWindHeadLeft[iSl]*fScaleSail, ySailPlanHeadLeft[iSl]*fScaleSail, NULL) ;
              LineTo (hdcMem, xSailWindHeadRight[iSl]*fScaleSail, ySailPlanHeadRight[iSl]*fScaleSail) ;
              LineTo (hdcMem, xSailWindFootRight[iSl]*fScaleSail, ySailPlanFootRight[iSl]*fScaleSail) ;
              LineTo (hdcMem, xSailWindFootLeft[iSl]*fScaleSail, ySailPlanFootLeft[iSl]*fScaleSail) ;
              CloseFigure (hdcMem) ;
              EndPath (hdcMem) ;
              StrokeAndFillPath (hdcMem) ;
              }
            }
          }
        }

      // redraw mast if sails not taken aback
      if (bBrig)
        {
        if ((iSl==4) && (sin(((90.0 - fSailAngle[iSl]) - fAppWindAngle) * fDegToRad) >= 0.0))    // foreyard drawing
          DrawMast(hdcMem,0,fFactor) ;  // fore mast
        if ((iSl==9) && (sin(((90.0 - fSailAngle[iSl]) - fAppWindAngle) * fDegToRad) >= 0.0))    // mainyard drawing
          DrawMast(hdcMem,1,fFactor) ;  // main mast
        }
       else
        {
        if ((iSl==3) && (sin(((90.0 - fSailAngle[iSl]) - fAppWindAngle) * fDegToRad) >= 0.0))    // foreyard drawing
          DrawMast(hdcMem,0,fFactor) ;  // fore mast
        if ((iSl==7) && (sin(((90.0 - fSailAngle[iSl]) - fAppWindAngle) * fDegToRad) >= 0.0))    // mainyard drawing
          DrawMast(hdcMem,1,fFactor) ;  // main mast
        if ((iSl==10) && (sin(((90.0 - fSailAngle[iSl]) - fAppWindAngle) * fDegToRad) >= 0.0))   // mizzenyard drawing
          DrawMast(hdcMem,2,fFactor) ;  // main mast
        }
      }
    SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;

    // draw bow and stern
    fTemp = fShipLength*fScaleSail/2.0 * fFactor;
    if ( fabs(fAppWindAngle) <90 )
      {
      ShowBow (hdcMem, fTemp) ;
      ShowStern (hdcMem, fTemp) ;
      }
    else
      {
      ShowStern (hdcMem, fTemp) ;
      ShowBow (hdcMem, fTemp) ;
      }

    // draw barricade
    if ( bBrig && fabs(fAppWindAngle) < 90 )
      {
      SelectObject(hdcMem, (HBRUSH) GetStockObject(WHITE_BRUSH)) ;
      fTemp = -((fShipLength/2.0)-fQDeckLength)*fScaleSail * fFactor; // quarterdeck
      fStep=fShipWidth*0.4*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
      BeginPath(hdcMem) ;
      MoveToEx (hdcMem, fTemp-fStep, fGunDeckHeight*fScaleSail, NULL) ;
      LineTo (hdcMem, fTemp-fStep, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fScaleSail) ;
      LineTo (hdcMem, fTemp+fStep, fGunDeckHeight*fScaleSail) ;
      CloseFigure(hdcMem) ;
      EndPath(hdcMem) ;
      StrokeAndFillPath(hdcMem) ;
      }
    }

  // Copy from memory to child window
  pPoint.x = 0 ;
  pPoint.y = 0 ;
  DPtoLP (hdcMem, &pPoint, 1) ;
  xStart = pPoint.x ;
  pPoint.x = cxChildPixels ;
  pPoint.y = cxChildPixels ;
  DPtoLP (hdcMem, &pPoint, 1) ;
  yStart = pPoint.y ;

  BitBlt (hdc,xStart,yStart,1000,1000,
          hdcMem, xStart,yStart, SRCCOPY) ;
  ReleaseDC (hwndChild[1], hdc) ;
  DeleteDC (hdcMem) ;
  }

/*-----------------------------------------
   ShowBow: Shown bow on perspective view    
   Version date: 99.09.19
-----------------------------------------*/
void ShowBow (HDC hdcMem, float fTemp) 
  {
  float fStep ;

  SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
  fStep=fShipWidth*0.5*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
  BeginPath(hdcMem) ;
  MoveToEx (hdcMem, fTemp-fStep, 0, NULL) ;
  LineTo (hdcMem, fTemp-fStep, fQDeckHeight*fScaleSail) ;
  LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fScaleSail) ;
  LineTo (hdcMem, fTemp+fStep, 0*fScaleSail) ;
  CloseFigure(hdcMem) ;
  EndPath(hdcMem) ;
  StrokeAndFillPath(hdcMem) ;
  if ( fabs(fAppWindAngle) >90. )
    {
    SelectObject(hdcMem, GetStockObject(WHITE_PEN));
    MoveToEx (hdcMem, fTemp, fQDeckHeight*fScaleSail, NULL) ;
    LineTo (hdcMem, fTemp, 0) ;
    }
  SelectObject(hdcMem, GetStockObject(BLACK_PEN));
  }

/*-----------------------------------------
   ShowStern: Shown stern on perspective view    
   Version date: 03.03.22
-----------------------------------------*/
void ShowStern (HDC hdcMem, float fTemp) 
  {
  int   iI ;
  POINT pDrawStern[4] ;
  float fStep ;

  SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
  fStep=fShipWidth*0.5*fScaleSail*cos(fAppWindAngle*fDegToRad) ;
  BeginPath(hdcMem) ;
  MoveToEx (hdcMem, -fTemp-fStep, 0, NULL) ;
  LineTo (hdcMem, -fTemp-fStep, fQDeckHeight*fScaleSail) ;
  LineTo (hdcMem, -fTemp+fStep, fQDeckHeight*fScaleSail) ;
  LineTo (hdcMem, -fTemp+fStep, 0*fScaleSail) ;
  CloseFigure(hdcMem) ;
  EndPath(hdcMem) ;
  StrokeAndFillPath(hdcMem) ;

  if ( fabs(fAppWindAngle) <90. )
    {
    SelectObject(hdcMem, hDeckBrush) ;
    for (iI=0; iI<4 ; iI++)
      {
      pDrawStern[iI].x=(pStern[iI].x*cos(fAppWindAngle * fDegToRad)*fScaleSail) - fTemp ;
      pDrawStern[iI].y=pStern[iI].y*fScaleSail ;
      }
    SelectObject(hdcMem, hDeckBrush) ;
    BeginPath (hdcMem) ;
    PolyBezier (hdcMem,pDrawStern,4) ;
    CloseFigure (hdcMem) ;
    EndPath (hdcMem) ;
    StrokeAndFillPath (hdcMem) ;
    SelectObject(hdcMem, (HPEN) hSeaBluePen) ;
    SelectObject(hdcMem, hSeaBrush) ;
    Rectangle (hdcMem, -1000, 0, 1000, -150) ;  // redraw sea over underwater stern
    SelectObject(hdcMem,GetStockObject(BLACK_PEN)) ;
    }
  }

/*-----------------------------------------
   SmallerJib: Sets smaller jib on frigate    
   Version date: 03.08.21
-----------------------------------------*/
void SmallerJib () 
  {
  HDC   hdc ;
  hdc = GetDC (hwnd) ;

  switch (iJibSet)
    {
    case 0:
    case 1:
      iJibSet=2 ;
      SetWindowText(hwndButton[7], "Working jib") ;
      SetWindowText(hwndButton[8], "Large jib") ;
      TextOut (hdc, xJib, yMessage+iSpacer, "Storm jib:      ",16) ;
      break ;
    case 2:
      iJibSet=1 ;
      SetWindowText(hwndButton[7], "Storm jib") ;
      SetWindowText(hwndButton[8], "Large jib") ;
      TextOut (hdc, xJib, yMessage+iSpacer, "Working jib:",12) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  SetFocus (hwndCheck[1]) ;
  NewJib () ;
  }

